@@ -38,10 +38,10 @@ $Id: definitions.h 2170 2013-09-19 12:08:27Z beerli $
 *----------------------------------------------------------------
 */
 #ifndef MIGRATEVERSION
-#define MIGRATEVERSION "4.1.4"
+#define MIGRATEVERSION "4.2.2a"
 #endif
 #ifndef MIGRATESUBVERSION
-#define MIGRATESUBVERSION "March-23-15"
+#define MIGRATESUBVERSION "July-18-15"
 #endif
 
 #define MAINTAINER "Peter Beerli <beerli@fsu.edu>"
@@ -178,13 +178,15 @@ $Id: definitions.h 2170 2013-09-19 12:08:27Z beerli $
 #define PDFOUTFILENUM 21
 #define SKYLINEFILE "skylinefile"
 #define SKYLINEFILENUM 23
-#define MYMAXFILENUM 24
+#define MYMAXFILENUM 28
 #define DIVFILE "divfile"
 #define DIVFILENUM 24
 #define TIPDATEFILE "datefile"
 #define TIPDATEFILENUM 25
 #define MIXFILE "mixfile"
 #define MIXFILENUM 26
+#define DIVTIMEFILENUM 27
+#define DIVTIMEFILE "divtimefile"
 #define BAYESNUMBIN 1500
 #define PRETTY_MAX 0
 #define PRETTY_P99 1
@@ -401,6 +403,8 @@ long chains is used */
 #define PI 3.14159265358979323846264338328 /*N[Pi,30]*/
 #define INV2PI 0.159154943091895335768883763373 /*N[1/(2 pi)]*/
 #define LOG2MINUSPIHALF -0.22579135264472743236 /*N[1/2(Log[2]-Log[Pi]),20]*/
+#define SQRT2PI 0.79788456080286535588 /*N[Sqrt[2/Pi], 20]*/
+#define SQRT2 1.4142135623730950488 /*N[Sqrt[2],20]]*/
 #define ROOTLENGTH         10000
 #define FIRSTCHAIN            -1
 #define FIRSTSTEP             -1
@@ -480,7 +484,7 @@ long chains is used */
 #define MINMIGSUMSTAT 0.000001
 /* the migration limit is per population, should be plenty
    for moderate sample sizes */
-#define MIGRATION_LIMIT    100000
+#define MIGRATION_LIMIT    10000
 #define SMALLEST_PROB     1e-100
 #define SMALLEST_GAMMA      1e-3
 #define BIGGEST_GAMMA        1e9
@@ -690,4 +694,13 @@ long chains is used */
 #define MYSYSTEM "Other"
 #endif
 
+#ifdef FASTERF
+#define ERF myerf
+#define ERFC myerfc
+#else
+#define ERF erf
+#define ERFC erfc
+#endif
+
+
 #endif /*definitions */
