@@ -67,9 +67,8 @@
 #include "tree.h"
 #include "world.h"
 #include "priors.h"
-#ifdef PRETTY
 #include "pretty.h"
-#endif /*PRETTY*/
+#include "speciate.h"
 #include <stdio.h>
 #ifdef BEAGLE
 #include "calculator.h"
@@ -84,6 +83,7 @@
 
 #ifdef GRANDCENTRAL
 #include <dispatch/dispatch.h>
+dispatch_semaphore_t semaphore;
 #endif
 
 //use this if you have nice() and need to be nice with others
@@ -91,11 +91,9 @@
 #include <unistd.h>
 #endif
 
-#ifdef PRETTY
 #ifdef WIN32
 #include "pretty-win32.h"
 #endif
-#endif
 
 
 /* Definitions for MCMCMC -- heated chains
@@ -180,7 +178,6 @@ sfmt_t ** sfmtH;
 // for bayesian output counter, initialized in bayes_init()
 long *mdimfilecount;
 
-//#ifdef PRETTY
 // for pretty printing
 pdf_doc doc;
 pdf_page page;
@@ -191,7 +188,6 @@ char pdf_time[LINESIZE+1];
 float page_height;
 float left_margin;
 float page_width;
-//#endif
 
 
 int
@@ -284,8 +280,7 @@ void
 print_heating_progress (world_fmt ** universe,
                         worldoption_fmt * options, long stepinc);
 
-long analyze_olddata (world_fmt * world, option_fmt * options, data_fmt * data, long *outfilepos);
-boolean analyze_oldbayesdata(world_fmt *world, option_fmt *options, data_fmt *data, long *outfilepos);
+boolean analyze_oldbayesdata(world_fmt **universe, option_fmt *options, data_fmt *data, long *outfilepos);
 void print_theta0(FILE *file, world_fmt *world, long maxreplicate);
 void profile_tables (option_fmt * options, world_fmt * world, long *gmaxptr);
 
@@ -338,6 +333,9 @@ static boolean EnableCoreDumps(void)
 }
 #endif
 
+
+
+
 ///
 /// the program migrate calculates migration rates and population sizes
 /// from genetic data, it allows for a wide variety of data types and many options
@@ -388,12 +386,10 @@ main (int argc, char **argv)
   
   //---------------------------------------------------------------------------------------------
   // windows specific code for pretty printing
-#ifdef PRETTY
 #ifndef MPI
 #ifdef WIN32
     set_haru_handler();
 #endif
-#endif
 #endif
 
     //---------------------------------------------------------------------------------------------
@@ -517,13 +513,80 @@ main (int argc, char **argv)
     //fprintf(stderr,"myID=%i myRepID=%i\n",myID, myRepID);    
     //---------------------------------------------------------------------------------------------
     // sampling phase
-    if (!options->readsum) // all go here except when reading old runs
+    if (!options->readsum && !options->checkpointing) // all go here except when reading old runs
       {
         run_sampler (options, data, universe, usize, &outfilepos, &Gmax);
       }
     else
       {
-	restarted_bayes_bool = analyze_oldbayesdata(EARTH, options, data, &outfilepos);
+	if(options->checkpointing)
+	  {
+	    // what was the last set of loci worked on (MPI?)
+	    long **unfinished = NULL;
+#ifdef MPI 
+	    long repmax = number_replicates2(options);
+#endif
+	    boolean recover_needed = FALSE;
+	    if (myID==MASTER)
+	      {
+		recover_needed = checking_bayesallfile(universe[0], options, data, &unfinished);
+		options->unfinished = unfinished;
+#ifdef MPI
+		long mysize = universe[0]->loci * repmax;
+		long buffer[3];
+		buffer[0]= (long) recover_needed;
+		buffer[1]= (long) EARTH->loci;
+		buffer[2]= (long) repmax;
+		MYMPIBCAST (&buffer[0], 3, MPI_LONG, MASTER, comm_world);
+		MYMPIBCAST (&(unfinished[0][0]), mysize, MPI_LONG, MASTER, comm_world);
+#endif
+	      }
+#ifdef MPI
+	    else
+	      {
+		long buffer[3];
+		MYMPIBCAST (&buffer[0], 3, MPI_LONG, MASTER, comm_world);
+		recover_needed = (boolean) buffer[0];
+		EARTH->loci = buffer[1];
+		repmax = buffer[2];
+		intvec2d(&unfinished, EARTH->loci,repmax);
+		long mysize = universe[0]->loci * repmax;
+		MYMPIBCAST (&(unfinished[0][0]), mysize, MPI_LONG, MASTER, comm_world);
+		options->unfinished = unfinished;
+#ifdef DEBUG
+		  printf("@Checking bayesallfile, to find last loci and replicates worked on\n");
+		  long maxsample1 = options->lsteps - 1;
+		  for (locus=0; locus < EARTH->loci; locus++)
+		    {
+		      long replicate;
+		      for (replicate=0; replicate < repmax; replicate++)
+			{
+			  long xx = unfinished[locus][replicate];
+			  printf("%5li ", xx);
+			} 
+		      printf("\n");
+		    }
+		  printf("[%li,%li]\n",maxsample1,mysize);		  
+#endif
+	      }
+#endif /*end MPI*/
+	    // set the last parameters like the options so that it can be restarted
+	    // this will be tricky for MPI, the master needs to seed the loci correctly
+	    if (TRUE) //recover_needed)
+	      {
+		run_sampler (options, data, universe, usize, &outfilepos, &Gmax);
+	      }
+	    else
+	      {
+	    	options->readsum = TRUE;
+	    	options->checkpointing = FALSE;
+	    	restarted_bayes_bool = analyze_oldbayesdata(universe, options, data, &outfilepos);
+	      }
+	  }
+	  else
+	    {
+	      restarted_bayes_bool = analyze_oldbayesdata(universe, options, data, &outfilepos);
+	    }
       }
     
     //---------------------------------------------------------------------------------------------
@@ -565,9 +628,7 @@ main (int argc, char **argv)
 	if(options->mighist || options->skyline)
 	  {
 	    print_event_values(EARTH);
-#ifdef PRETTY
 	    pdf_histogram_legend();
-#endif
 	  }              
 #ifdef UEP
 	// print UEP probabilities
@@ -603,17 +664,15 @@ main (int argc, char **argv)
 	// printing of MCMC run characteristics
 	fprintf(EARTH->outfile,"\n\nMCMC run characteristics\n");
 	fprintf(EARTH->outfile,"========================\n\n");
+	bayes_print_hyperprior(EARTH->outfile,EARTH);
 	bayes_print_accept(EARTH->outfile,EARTH);
-#ifdef PRETTY
+	pdf_bayes_print_hyperpriors(EARTH);
 	pdf_bayes_print_accept(EARTH);
-#endif
 	fprintf(EARTH->outfile,"\nAutocorrelation and Effective sample size\n");
 	fprintf(EARTH->outfile,"-------------------------------------------------------------------\n\n");
 	//	print_bayes_ess(EARTH->outfile,EARTH,EARTH->numpop2 + EARTH->bayes->mu * EARTH->loci + 1,2,EARTH->auto_archive, EARTH->ess_archive);
 print_bayes_ess(EARTH->outfile,EARTH,EARTH->numpop2 + EARTH->bayes->mu + 1, 2, EARTH->auto_archive, EARTH->ess_archive);
-#ifdef PRETTY
 	pdf_bayes_print_ess(EARTH);
-#endif	
 	if(strchr("aet",EARTH->options->burnin_autostop))
 	  {
 	    print_burnin_stop(EARTH->outfile, universe, options);
@@ -692,12 +751,14 @@ print_bayes_ess(EARTH->outfile,EARTH,EARTH->numpop2 + EARTH->bayes->mu + 1, 2, E
     MYMPIBARRIER (comm_world);
     MPI_Finalize ();
 #endif /*MPI*/
-    //	myfree(seed);
-    //myfree(newseed);
-    //myfree(generator);
-    //free_universe(universe, usize, options);
-    //destroy_data(data);
-    //destroy_options(options);
+    myfree(seed);
+    myfree(newseed);
+    myfree(generator);
+    destroy_data(data, EARTH);
+    free_universe(universe, usize, options);
+    myfree(data);
+    destroy_options(options);
+    myfree(options);
     // for debugging with MallocDebug on macosx
 #ifdef MACMALLOCDEBUG
     while(TRUE)
@@ -720,16 +781,25 @@ long  calculate_newpop_numpop(option_fmt *options, data_fmt *data)
   long numpop = data->numpop;
   //  char *strsep(char **, const char *);
 #ifdef WIN32
-    // dev studio does not understand the #warning preprocessor directive, it uses pragma but I am not ready for that yet
-    //#warning "Windows version is hardcoded to not have more than 5000 populations"
-    long temp[5000];
+  // (old?) windows compilers have difficult to assign memory 
+  // based on variables
+  long temp[5000];//this is __way__ more than migrate ever will be 
+                  // able to handle    
 #else
-  long temp[data->numpop];
+    long temp[data->numpop < options->newpops_numalloc ? options->newpops_numalloc : data->numpop];
 #endif
   //use order of populations {1,2,3,4,5,....} , do not start with zero!!
   //reset to options->newpops={1,1,2,1,2,3,4,...}
-  if(options->newpops_numalloc < numpop)
+  if (options->newpops_numalloc != numpop)
     {
+      // newpops_numalloc > numpop ===> warn and resize
+      // newpops_numalloc < numpop ===> resize and fill
+      if(options->newpops_numalloc > numpop)
+	{
+	  warning("Population number in datafile is smaller than in the relabel option!");
+	  warning("Relabel option will be adjust to number of populations in datafile");
+	  options->newpops_numalloc = numpop;
+	}
       options->newpops = (long *) myrealloc(options->newpops,sizeof(long)*numpop);
       for(i=options->newpops_numalloc;i<numpop;i++)
 	{
@@ -737,8 +807,8 @@ long  calculate_newpop_numpop(option_fmt *options, data_fmt *data)
 	}
       options->newpops_numalloc = numpop;
     }
-  memcpy(temp,options->newpops,sizeof(long)*options->newpops_numalloc);
-  qsort((void *) temp, (size_t) options->newpops_numalloc, sizeof(long), longcmp);
+  memcpy(temp,options->newpops,sizeof(long)*numpop);
+  qsort((void *) temp, (size_t) numpop, sizeof(long), longcmp);
   pop = temp[0];
   newnumpop = 1;
   for(i=1;i<numpop;i++)
@@ -860,8 +930,11 @@ run_sampler (option_fmt * options, data_fmt * data, world_fmt ** universe,
   init_world (EARTH, data, options);
   //  set_meanmu(EARTH,options);
   if(EARTH->cold)
-    single_chain_var (EARTH, 0L, &var, NULL, NULL);
-  
+    {
+      //if(EARTH->has_speciation)
+      //EARTH->divtimefile = fopen("divtimefile","w");
+      single_chain_var (EARTH, 0L, &var, NULL, NULL);
+    }
 #ifdef NEXUSTREE
   if (EARTH->options->treeprint == LASTCHAIN)
     {
@@ -883,9 +956,7 @@ run_sampler (option_fmt * options, data_fmt * data, world_fmt ** universe,
       if (options->progress)
 	print_data_summary (stdout, EARTH, options, data);
       /* print to outfile */
-#ifdef PRETTY
       pdf_master_init(EARTH, options, data);
-#endif 
       *outfilepos = print_title (EARTH, options);
       print_options (EARTH->outfile, EARTH, options, data);
       print_data_summary (EARTH->outfile, EARTH, options, data);
@@ -1001,7 +1072,7 @@ run_loci (world_fmt ** universe, int usize, option_fmt * options,
 	  run_locus (universe, usize, options, data,
 		     localheating_pool, maxreplicate, locus, treefilepos, Gmax);
 	}
-      //free_datapart(data,universe[0],locus);
+      free_datapart(data,universe[0],locus);
     }
 }
 
@@ -1233,8 +1304,9 @@ print_heating_progress2 (FILE * file, worldoption_fmt * options,
 }
 
 
-boolean analyze_oldbayesdata(world_fmt *world, option_fmt *options, data_fmt *data, long *outfilepos)
+boolean analyze_oldbayesdata(world_fmt **universe, option_fmt *options, data_fmt *data, long *outfilepos)
 {
+  world_fmt *world = universe[0];
 #ifndef MPI
   long pop;
 #endif
@@ -1289,10 +1361,7 @@ boolean analyze_oldbayesdata(world_fmt *world, option_fmt *options, data_fmt *da
   read_bayes_fromfile(world->bayesmdimfile, world, options, files, numfiles);
 #endif
   //set_meanmu(world,options);
-
-#ifdef PRETTY
   pdf_master_init(world, options, data);
-#endif
   for(locus=0;locus<world->loci;locus++)
     {
       if(world->data->skiploci[locus])
@@ -1350,6 +1419,10 @@ setup_locus (long locus, world_fmt * world, option_fmt * options,
     print_menu_locus (world->options->logfile, world, locus);
   world->start = TRUE;
 #ifdef NEWVERSION
+  if(options->has_unassigned && locus>0)
+    {
+      reset_all_assigned_nodes(world);
+    }
   if(options->haplotyping)
     {
       world->data->haplotyping = TRUE;
@@ -1539,6 +1612,10 @@ heating_prepare (world_fmt ** universe, int usize,
 	klone_mutationmodel(universe[chain],EARTH, data, locus);
         klone (EARTH,universe[chain], options, data, options->heat[chain]);
 #ifdef NEWVERSION
+	if(options->has_unassigned && locus>0)
+	  {
+	    reset_all_assigned_nodes(universe[chain]);
+	  }
 	if(options->haplotyping)
 	  {
 	    universe[chain]->data->haplotyping = TRUE;
@@ -1772,12 +1849,14 @@ run_updates (world_fmt ** universe,
 #else /*heating but not using threads */
 #ifdef GRANDCENTRAL
         //dispatch_queue_t queue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0);
+	semaphore = dispatch_semaphore_create(1);
         dispatch_apply(options->heated_chains, queue,
                        ^(unsigned long ii) {
 			 sfmtp = sfmtH[ii];
                          run_one_update (universe[ii]);
                        }
                        );
+	dispatch_release(semaphore);
 #else
 	//count++;
 	run_one_update (universe[0]);
@@ -1847,6 +1926,8 @@ run_increments (world_fmt ** universe,
     }
     if (EARTH->options->treeprint != _NONE)
       print_tree (EARTH, EARTH->G, &treefilepos);
+    if(EARTH->has_speciation && options->recorddivtime)
+      record_parameters(EARTH);
 }
 
 /// \brief run a chain
@@ -1866,14 +1947,33 @@ run_steps (world_fmt ** universe,
 #ifdef MPI
     // does not work? check_memory_limit();
 #endif
-    for (step = 0; step < steps; step++)// Cesky 2013
+    long start = 0;
+    if (options->checkpointing)
+      start = options->unfinished[EARTH->locus][rep];
+#ifndef MPI
+    const boolean progress = EARTH->options->progress && EARTH->cold;
+    long delta = ((steps > 100000) ? (steps / 10) : (steps/3));
+    boolean reportdone=FALSE;
+    long starttime = expected_end_burnin(EARTH,0.0, 0, " ");
+    if (delta==0)
+      delta=1;
+#endif
+    for (step = start; step < steps; step++)
     {
 #ifdef MPI
-      // does not work?? check_memory_limit();
+      // check_memory_limit() does not work;
+#else
+      // too much writing for MPI therefore we do not report progress on burnin
+      if (!reportdone && progress && (step+1) % delta == 0)
+	{
+	  reportdone=TRUE;
+	  expected_end_burnin(EARTH, 100.0*(step-1.0)/steps, starttime,"Sampling complete ");
+	}
 #endif
         EARTH->increment = increment;
         run_increments (universe, usize, options, localheating_pool,
                         increment, step, steps);
+	
 #ifdef UEP
         store_uep (EARTH);
 #endif
@@ -2036,6 +2136,12 @@ run_replicate (long locus,
     EARTH->repkind = SINGLECHAIN;
     EARTH->replicate = replicate;
 
+    if (options->checkpointing)
+      {
+	long start = options->unfinished[locus][replicate];
+	if (start >= options->lsteps - 1)
+	  return;
+      }
     if (setup_locus (locus, EARTH, options, data) == 0)
         return;
     if (options->heating)
@@ -2143,10 +2249,8 @@ void print_burnin_stop(FILE *file, world_fmt **universe, option_fmt * options)
       //			    world->burnin_stops[z].worker = myID;
     }
   fprintf(file,"(*=worst)\n\n");
-#ifdef PRETTY
   if(file==EARTH->outfile)
     pdf_burnin_stops(EARTH, maxreplicate);
-#endif
 }
 
 MYREAL combine_scaling_factor(world_fmt *world)
@@ -2262,6 +2366,7 @@ void print_bayesfactor(FILE *file, world_fmt **universe, option_fmt * options)
   MYREAL asum = 0.;
   MYREAL lsum;
   MYREAL lsum0;
+  MYREAL ratio;
   MYREAL scaling_factor = 0.0;
   MYREAL *locusweight = EARTH->data->locusweight;//invariant loci treatment
   // calculate the harmonic mean score
@@ -2274,13 +2379,12 @@ void print_bayesfactor(FILE *file, world_fmt **universe, option_fmt * options)
 
   fprintf(file,"\n\n\nLog-Probability of the data given the model (marginal likelihood = log(P(D|thisModel))\n");
   fprintf(file,"--------------------------------------------------------------------\n[Use this value for Bayes factor calculations:\nBF = Exp[log(P(D|thisModel) - log(P(D|otherModel)]\nshows the support for thisModel]\n\n");
-#ifdef PRETTY
   if(file==EARTH->outfile)
     pdf_bayes_factor_header(EARTH,options);
-#endif
   if(EARTH->loci>1)
     {
-      fprintf(file,"\n\nLocus      Raw Thermodynamic score(1a)  Bezier approximated score(1b)     Harmonic mean(2)\n");
+      //      fprintf(file,"\n\nLocus      Raw Thermodynamic score(1a)  Bezier approximated score(1b)     %s\n",options->heating ?  "Stepstone score(2)" : " Harmonic mean(2)");
+      fprintf(file,"\n\nLocus      Raw Thermodynamic score(1a)  Bezier approximated score(1b)     %s\n"," Harmonic mean(2)");
       fprintf(file,"------------------------------------------------------------------------------------------\n");
       if(file==EARTH->outfile)
 	pdf_bayes_factor_rawscores_header(EARTH,options);
@@ -2296,11 +2400,13 @@ void print_bayesfactor(FILE *file, world_fmt **universe, option_fmt * options)
       //  printf("\n");    	  
       //}
 #endif
+      //hsum = 0.0;
       for(locus=0;locus<EARTH->loci;locus++)
 	{
 	  if(EARTH->data->skiploci[locus])
 	    continue;
-	  
+
+	  //ratio = 0;
 	  lsum = 0.;
 	  lsum0 = 0.;
 	  // from very hot to cold, the 
@@ -2334,6 +2440,8 @@ void print_bayesfactor(FILE *file, world_fmt **universe, option_fmt * options)
 	      //Simpson's rule and trapezoidal are the same when I only have function values at a and b
 	      val0 = locusweight[locus] * EARTH->bf[locus * hc + t-1];
 	      val1 = locusweight[locus] * EARTH->bf[locus * hc + t];
+	      // ratio += val1 - val0;
+
 	      //we keep last element to adjust for Bezier approximation
 	      lsum0 = (heat0 - heat1) * (val0 + val1) * 0.5;
 	      lsum += lsum0; 
@@ -2361,13 +2469,15 @@ void print_bayesfactor(FILE *file, world_fmt **universe, option_fmt * options)
 	  //#ifdef DEBUG
 	  //	  fprintf(stdout,"Thermo[%li]=(%f) - (%f) + (%f) = (%f)\n",locus, lsum, lsum0, approxlsum, approxlsum + lsum - lsum0);
 	  //	  #endif
+
+	  //EARTH->hmscale[locus]=ratio;
+	  //EARTH->hm[locus] = 1.0;
+	  //hsum += ratio;
 	  if(EARTH->loci>1)
 	    {
 	      fprintf(file,"  %5li%22.2f        %22.2f           %12.2f\n", locus + 1, lsum, lsum-lsum0+approxlsum, EARTH->hmscale[locus] - log (EARTH->hm[locus])); 
-#ifdef PRETTY
 	      if(file==EARTH->outfile)
 		pdf_bayes_factor_rawscores(locus, lsum, lsum-lsum0+approxlsum, EARTH->hmscale[locus] - log (EARTH->hm[locus]));
-#endif
 	    }
 	  bfsum2 += approxlsum + lsum - lsum0;  	  
 	  bfsum += lsum; //+ EARTH->bfscale[locus];
@@ -2381,10 +2491,8 @@ void print_bayesfactor(FILE *file, world_fmt **universe, option_fmt * options)
 	  hsum   += scaling_factor;
 	  fprintf(file,"---------------------------------------------------------------------------------------\n");
 	  fprintf(file,"  All  %22.2f        %22.2f           %12.2f\n[Scaling factor = %f]\n", bfsum, bfsum2, hsum, scaling_factor); 
-#ifdef PRETTY
 	  if(file==EARTH->outfile)
 	    pdf_bayes_factor_rawscores(-1L, bfsum, bfsum2, hsum);
-#endif
 	}
       else 
 	{
@@ -2403,10 +2511,8 @@ void print_bayesfactor(FILE *file, world_fmt **universe, option_fmt * options)
 	  if(EARTH->loci>1)
 	    {
 	      fprintf(file,"    %5li           ------                           ------            %12.2f\n", locus+1, EARTH->hmscale[locus] - log (EARTH->hm[locus])); 
-#ifdef PRETTY
 	      if(file==EARTH->outfile)
 		pdf_bayes_factor_rawscores_harmo(locus, EARTH->hmscale[locus] - log (EARTH->hm[locus]));
-#endif
 	    }
 	}
       if(EARTH->loci>1)
@@ -2415,10 +2521,8 @@ void print_bayesfactor(FILE *file, world_fmt **universe, option_fmt * options)
 	  hsum += scaling_factor;
 	  fprintf(file,"---------------------------------------------------------------------------------------\n");
 	  fprintf(file,"      All           ------                           ------            %12.2f\n", hsum); 
-#ifdef PRETTY
 	  if(file==EARTH->outfile)
 	    pdf_bayes_factor_rawscores_harmo(-1L, hsum);
-#endif
 	}
       else
 	{
@@ -2429,12 +2533,10 @@ void print_bayesfactor(FILE *file, world_fmt **universe, option_fmt * options)
 e of 1.0 to at least 100,000.\n\n");
 	}
     }
-#ifdef PRETTY
   if(file==EARTH->outfile)
     {
       pdf_bayes_factor(EARTH, bfsum, bfsum2, hsum, asum, maxreplicate,scaling_factor);
     }
-#endif
 }
 
 ///
@@ -2533,7 +2635,9 @@ void calculate_BF(world_fmt **universe, option_fmt *options)
   //stored in the cold chain
   if(options->heating)
     {
-      //printf("%i>BF: %li*4*i:",myID, locus); 
+#ifdef DEBUG
+      printf("%i>BF: %li*4*i:",myID, locus);
+#endif 
       for (i = 0; i < hc; i++)
 	{
 	  xx = universe[i]->likelihood[universe[i]->G];
@@ -2543,9 +2647,13 @@ void calculate_BF(world_fmt **universe, option_fmt *options)
 	    {
 	      warning("am or likelihood failed: am=%f",myID, locus, i,EARTH->am[locus]);
 	    }
-	  //printf("%f ",EARTH->bf[locus * hc + i]); 
+#ifdef DEBUG
+	  printf("%f ",EARTH->bf[locus * hc + i]); 
+#endif
 	}
-      //printf("\n");
+#ifdef DEBUG
+      printf("\n");
+#endif
     }
 }
 
