@@ -87,7 +87,7 @@ void chooseOrigin (proposal_fmt * proposal);
 void construct_localtimelist (timelist_fmt * timevector,
                               proposal_fmt * proposal);
 void traverseAllNodes (node * theNode, node *** nodelist, long *node_elem,
-                       long *oldnode_elem, int include_migration);
+                       long *oldnode_elem, int include_migration, div_fmt **divlist);
 void chooseTarget (proposal_fmt * proposal, timelist_fmt * timevector,
                    node ** bordernodes, long *bordernum);
 void findbordernodes (node * theNode, proposal_fmt * proposal, long pop,
@@ -182,348 +182,12 @@ return 1 if tree was accepted, 0 otherwise
 assign is set for assigning individuals, caller is responsible to reset the 
 origin back to the original state when fail
 */
-#ifdef BEAGLEDEBUG
-    int beagle_control=1;
-#endif
-    //short show=0;
-    boolean coalesced;
-    //  boolean test;
-    char event;
-    long slider;
-    long bordernum;
-    long actualpop = -99, zz;
-    long skyz=0;
-    MYREAL endtime, nexttime, age;
-    long newpop;
-    long oldpop = -99; //to keep clang static analyzer happy and crash when used like this
-#ifdef UEP    
-    boolean uepsuccess = FALSE;
-#endif
-#ifdef TESTING2
-    proposal_fmt *proposal = world->proposal; 
-#else
-    proposal_fmt *proposal=NULL; 
-#endif
-    /*scratchpad  in which all involved nodes
-     are recorded and help arrays, e.g. migration arrays,
-     are stored */
-    timelist_fmt *timevector = NULL; /* local timelist */
-    vtlist *tentry = NULL; /*pointer into timeslice */
-    
-    // check whether there is something to assign
     if (assign && world->unassignednum<2)
       return 0;
-    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
-    //if (world->has_speciation)
-    //  {
-	return newtree_update(world,g,assign);
-	//  }
-	//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
-    /* ---------------------------------------------------------
-     initialize local timelist and construct residual timelist 
-     find the start node and snip of the branch and node below */
-    new_localtimelist (&timevector, &world->treetimes[0], world->numpop);
-#ifdef TESTING2
-    if(world->has_proposal_details)
-        reset_proposal (&proposal, world);//, &world->treetimes[0], world);
-    else
-    {
-    	set_new_proposal (&proposal, &world->treetimes[0], world);
-    	world->has_proposal_details = TRUE;
-    }
-#else
-    new_proposal (&proposal, &world->treetimes[0], world);
-#endif
-    if(assign)
-      {
-	if(chooseUnassigned (proposal)==FAILURE)
-	  {
-	    free_timevector (timevector);
-#ifndef TESTING2
-	    free_masterproposal (proposal);
-#endif
-	    return 0;
-	  }
-	oldpop = proposal->origin->actualpop;
-	newpop = oldpop;
-	if (world->numpop>1)
-	  {
-	    while (newpop == oldpop)
-	      {
-		newpop = RANDINT(0,world->numpop-1);
-	      }
-	  }
-	reassign_individual(world,proposal->origin,newpop);
-      }
-    else
-      {
-	chooseOrigin (proposal);
-      }
-    //
-    if(world->options->bayes_infer)
-    {
-        world->bayes->starttime = -proposal->origin->tyme; // we start with negative value to
-        // indicate that we do not know yet whether we will accept this move or not
-        // if the move is accepted then the time will be negated (->positiv), we record
-        // all start and stop times one could easily imagine that some parameter values
-        // never gets accepted by the genealogy, and when we record this for bayesallfile we would
-        // like to know this. see further doen at the acceptance for the proposals
-    }
-    //
-    construct_localtimelist (timevector, proposal);
-    tentry = &(*timevector).tl[0];
-    age = proposal->origin->tyme;
-    if(age<0.0)
-      {
-	error("Abort time was negative");
-      }
-    zz = 0;
-    while ((tentry->age < age || tentry->age - age < SMALLEPSILON)&& zz < (*timevector).T)
-    {
-        tentry = &(*timevector).tl[zz];
-        zz++;
-    }
-    zz--;
-    ///
-
-    skyz=world->timeelements-1;
-    while (skyz > 0 && tentry->age <  proposal->world->times[skyz])
-      skyz-- ;
-    skyz++;
-    nexttime =  proposal->world->times[skyz];
-    tentry->timeslice=skyz-1;
-    ///
-    if (tentry->age < nexttime)
-      nexttime = tentry->age;
-
-    if ((*timevector).T > 1)
-        endtime = (*timevector).tl[(*timevector).T - 2].age;
-    else
-        endtime = 0.0;
-    proposal->time = 0.0;
-#ifdef TREEDEBUG
-    //    fprintf(stdout,"%i> begin zz=%li nextime=%f (pt=%f+age=%f)=%f,pot=%f\n",myID, zz, nexttime, proposal->time, age, age+proposal->time,proposal->origin->tyme);
-#endif
-    proposal->time = age;
-    coalesced = FALSE;
-    /*------------------------------------
-     main loop: sliding down the tree  */
-    slider = 0;
-#ifdef DEBUG
-    //printf("\nTime    Pop Type Lineages [mutrate=%f]\n",proposal->world->options->mu_rates[world->locus]);
-#endif
-    while (nexttime <= endtime)
-    {
-        actualpop =
-        (proposal->migr_table_counter >
-         0) ? proposal->migr_table[proposal->migr_table_counter -
-                                   1].from : proposal->origin->pop;
-#ifdef TREEDEBUG
-	//	if(age == 0.0 && proposal->origin->tyme>0.0)
-	//  {
-	//    printf("\n\n%f\n\n",age);
-	//  }
-#endif
-        proposal->time = age + eventtime (proposal, actualpop, tentry, &event);
-#ifdef TREEDEBUG
-       fprintf(stdout,"@@%i> zz=%li (pt=%f+age=%f)=%f,pot=%f\n",myID, zz, proposal->time-age, age, proposal->time,proposal->origin->tyme);
-#endif
-	if(proposal->time < 0.0 || isnan(proposal->time))
-	  {
-	    error("abort");
-	  }
-        //	age = proposal->time;
-        
-        if(proposal->time < proposal->origin->tyme)
-        {
-	  fprintf(stdout,"%i> Proposal failed because of unordered entry in time list, abort this sample\nProposed time=%f origin time=%f nexttime=%f, deltatime=%f\n", myID, proposal->time, proposal->origin->tyme, nexttime, age);
-            // we end up here when the migration events exceed the upper limit
-	  loopcleanup(assign,world,proposal,oldpop,timevector);
-	  return 0;
-            //	    error("Proposal of time failed\n");
-        }
-        if (proposal->time < nexttime)
-        {
-            if (event == 'm' || event == 'd')
-            {
-	      if (!migrate (proposal, proposal->origin, event))
-                {
-                    // we end up here when the migration events exceed the upper limit
-		  loopcleanup(assign,world,proposal,oldpop,timevector);
-		  return 0;
-                }
-                age = proposal->time;
-                continue;
-            }
-            else
-            {   /*coalesce */
-                chooseTarget (proposal, timevector, proposal->bordernodes,
-                              &bordernum);
-                if(bordernum == 0)
-                {
-		  if (!migrate (proposal, proposal->origin, event))
-                    {
-                        // we end up here when the migration events exceed the upper limit
-		      loopcleanup(assign,world,proposal,oldpop,timevector);
-		      return 0;
-                    }
-                    age = proposal->time;
-                    continue;
-                    //		    warning("chooseTarget failed at age %f\n", proposal->time);
-                    //		    free_proposal (proposal);
-                    //		    free_timevector (timevector);
-                    //		    return 0;
-                }
-                pretendcoalesce1p (proposal);
-#ifdef UEP
-                uepsuccess = is_success_pseudo_uep (proposal);
-#endif
-                
-                coalesced = TRUE;
-                break;
-            }
-        }   /*end if proposal->time < nextime */
-        age = nexttime;
-#ifdef TREEDEBUG
-	//	if(age < proposal->origin->tyme)
-	//  {
-	//    printf("darn: nextime=%f\n",nexttime);
-	//  }
-#endif
-        zz++;
-        if(zz >= timevector->T)
-        {
-            break;
-        }
-        tentry = &(*timevector).tl[zz]; /*next entry in timelist */
-	// skyparam
-	if (tentry->age > proposal->world->times[skyz])
-	  {
-	    zz--;
-	    tentry = &(*timevector).tl[zz]; /*next entry in timelist */
-	    tentry->timeslice = skyz;
-	    nexttime =  proposal->world->times[skyz];
-	    skyz++;
-	  }
-	else
-	  {
-	    nexttime = tentry->age;
-	    tentry->timeslice = skyz-1;
-	  }
-    }
-    if (!coalesced)
-    {
-        if (!pre_population(proposal, (*timevector).tl, (*timevector).T - 1, &slider))
-        {
-	  loopcleanup(assign,world,proposal,oldpop,timevector);
-	  return 0;
-        }
-        //	fprintf(stderr,"%i> last time=%f, end time = %f (%c)\n", myID, age,(*timevector).tl[(*timevector).T-1].eventnode->tyme,(*timevector).tl[(*timevector).T-1].eventnode->type );
-        pretendcoalesce1p (proposal);
-#ifdef UEP
-        
-        uepsuccess = is_success_pseudo_uep (proposal);
-#endif
-        
-    }
-    if (
-#ifdef UEP
-        ((!world->options->uep && !uepsuccess)
-         || (world->options->uep && uepsuccess)) &&
-#endif
-        (acceptlike (world, proposal, g, timevector)))
-      {
-#ifdef BEAGLE
-	//	printf("Accepted\n");
-	change_beagle(world->root->next->back,world->beagle,world->sumtips);
-	change_beagle_scalingIndex(world->beagle);
-#endif
-        if (proposal->time > world->root->tyme)
-        {   /*saveguard */
-            world->root->tyme += proposal->time;
-        }
-        coalesce1p (proposal);
-	/*	if (world->root->next->back->pop!=2)
-	  {
-	    warning("Root is not the ancenstor");
-	    }*/
-	if(world->cold && assign)
-	  {
-	    //	    fprintf(stdout, "Proposal succeeded: %s %li  was changed to %li\n", proposal->origin->truename, oldpop, proposal->origin->actualpop);
-	    record_assignment(world->locus, world);
-	  }
-        //DEBUG
-        //traverse_check(crawlback (proposal->root->next));printf("*");
-#ifdef DEBUG
-        //traverse_adjust(world->root->next->back, 1.0);
-#endif
-        // record the time interval that was used for the lineage.
-        if(world->options->bayes_infer)
-        {
-            world->bayes->starttime = -world->bayes->starttime;
-            world->bayes->stoptime = proposal->time;
-            world->treelen = 0.0;
-            calc_treelength (world->root->next->back, &world->treelen);
-        }
-        //
-#ifdef UEP
-        world->likelihood[g] = treelikelihood (world);
-        if (world->options->uep)
-        {
-            update_uep (world->root->next->back, world);
-            check_uep_root (world->root->next->back, world);
-            world->treelen = 0.0;
-            calc_treelength (world->root->next->back, &world->treelen);
-            world->ueplikelihood = ueplikelihood (world);
-            world->likelihood[g] = /*world->ueplikelihood +*/ world->likelihood[g];
-        }
-#else
-#ifdef BEAGLE
-	//set_beagle_dirty(proposal->origin,proposal->target,showtop(world->root->next->back));
-	//reset_beagle(world->beagle);
-	//smooth (world->root->next, crawlback(world->root->next), world, world->locus);
-	//if(world->beagle->numbranches>0)
-	//  {
-	world->likelihood[g] = proposal->likelihood; //treelikelihood (world);
-#ifdef BEAGLEDEBUG
-	if(beagle_control==1)// && fabs(proposal->likelihood - world->likelihood[g])>EPSILON)
-	  {
-	    printf("Recalculate LnL using treelikelihood()\n");
-	    world->likelihood[g] = force_beagle_recalculate(world,world->locus);
-	    printf("%i> %f = newLnL=%f + recalcLnL=%f\n",myID, proposal->likelihood - world->likelihood[g], proposal->likelihood,world->likelihood[g]);
-	    debug_beagle(proposal->world->beagle);
-	  }
-#endif
-#else
-        world->likelihood[g] = treelikelihood (world);
-#endif /*BEAGLE*/
-#endif /*UEP*/
-        /* create a new timelist */
-        construct_tymelist (world, &world->treetimes[0]);
-        //        if (world->options->treeprint != _NONE && world->cold)
-        //  print_tree (world, g, &treefilepos);
-        world->migration_counts = 0;
-        /* report the number of migration on the tree */
-        count_migrations (world->root->next->back, &world->migration_counts);
-
-#ifndef TESTING2
-        free_masterproposal (proposal);
-#endif
-        free_timevector (timevector);
-        return 1;   /* new tree accepted */
-      }
-    else
-      {
-        // record the time interval that was used for the lineage.
-        if(world->options->bayes_infer)
-        {
-            world->bayes->stoptime = -proposal->time;
-        }
-	loopcleanup(assign,world,proposal,oldpop,timevector);
-      }
-    return 0;   /* not accepted */   
+    return newtree_update(world,g,assign);
 }
 
+
 #ifdef SLATKIN_IMPORTANCE
 long slatkin_importance(world_mt *world, long g)
 {
@@ -549,12 +213,14 @@ long slatkin_importance(world_mt *world, long g)
 /// lineages have a large storage device that is accessed from the 
 /// tl[i].lineages.
 void
-new_localtimelist (timelist_fmt ** ntl, timelist_fmt * otl, long numpop)
+new_localtimelist (timelist_fmt ** ntl, timelist_fmt * otl, 
+long numpop)
 {
-    //    long i;
     (*ntl) = (timelist_fmt *) mycalloc (1, sizeof (timelist_fmt));
     (*ntl)->tl = (vtlist *) mymalloc ((*otl).allocT * sizeof (vtlist));
     (*ntl)->allocT = otl->allocT;
+    //(*ntl)->allocT = otl->T+2;
+    //printf("%i> alloc new_localtimelist: %li\n",myID,(*ntl)->allocT);
     (*ntl)->T = otl->T;
     (*ntl)->oldT = otl->oldT;
     //memcpy ((*ntl)->tl, otl->tl, otl->allocT * sizeof (vtlist));
@@ -588,25 +254,20 @@ new_localtimelist_new (timelist_fmt ** ntl, timelist_fmt * otl, long numpop)
 void
 new_proposal (proposal_fmt ** proposal, timelist_fmt * tl, world_fmt * world)
 {
+  const long np = world->numpop2 + world->bayes->mu + world->species_model_size * 2 ;
 #ifdef UEP    
     long j;
 #endif
-    //    long mal = world->data->maxalleles[world->locus];
-    //long listsize = ((*tl).allocT + (*tl).T + 5);
     long listsize = 2*(world->sumtips + 2);
     long sumtips = world->sumtips;
     mutationmodel_fmt *s;
-    //long endsite;
     // allocate the scratchpad (contains a shadow of the tree)
     (*proposal) = (proposal_fmt *) mycalloc (1, sizeof (proposal_fmt));
     (*proposal)->likelihood = -HUGE;
     // pointers and values to outside structures
     (*proposal)->world = world;
-    //(*proposal)->datatype = world->options->datatype;
     (*proposal)->sumtips = world->sumtips;
     (*proposal)->numpop = world->numpop;
-    //(*proposal)->endsite = world->data->seq[0]->endsite;
-    //(*proposal)->fracchange = world->data->seq[0]->fracchange;
     (*proposal)->param0 = world->param0;
     (*proposal)->root = world->root;
     (*proposal)->migration_model = world->options->migration_model;
@@ -614,21 +275,24 @@ new_proposal (proposal_fmt ** proposal, timelist_fmt * tl, world_fmt * world)
     (*proposal)->mig0list = world->mig0list;
     (*proposal)->design0list = world->design0list;
     
-    (*proposal)->listsize =  listsize;
-    
+    (*proposal)->aboveorigin_allocsize = listsize;    
     // nodes above the picked node + migration nodes
-    (*proposal)->aboveorigin =
-    (node **) mycalloc (listsize, sizeof (node *));
-    
+    (*proposal)->aboveorigin = (node **) mycalloc (listsize, sizeof (node *));
+    // list to record divergences above picked node
+    (*proposal)->divlist = (div_fmt *) mycalloc (1, sizeof (div_fmt));
+    (*proposal)->divlist->divlist = (longpair *) mycalloc (listsize, sizeof (longpair));
+    (*proposal)->divlist->div_allocsize = listsize;
+    (*proposal)->param0save = (MYREAL *) mycalloc (np, sizeof (MYREAL));
     // node data holding vector for bordernodes, line_f, line_t
-    (*proposal)->nodedata =
-    (node **) mycalloc (3 * listsize, sizeof (node *));
+    //(*proposal)->nodedata =
+    //(node **) mycalloc (3 * listsize, sizeof (node *));
     // adjacent nodes of the picked nodes
-    (*proposal)->bordernodes = (*proposal)->nodedata;
-    // line_f nodes
-    (*proposal)->line_f =   (*proposal)->bordernodes + listsize;
-    (*proposal)->line_t =  (*proposal)->line_f + listsize;
-    
+    (*proposal)->bordernodes_allocsize = listsize;    
+    (*proposal)->bordernodes = (node **) mycalloc (listsize, sizeof (node *)); 
+    (*proposal)->line_f_allocsize = listsize;    
+    (*proposal)->line_f =      (node **) mycalloc (listsize, sizeof (node *));
+    (*proposal)->line_t_allocsize = listsize;    
+    (*proposal)->line_t =      (node **) mycalloc (listsize, sizeof (node *));
     (*proposal)->mf = (MYREAL **) calloc(world->numsubloci[world->locus],sizeof(MYREAL*));
     (*proposal)->mt = (MYREAL **) calloc(world->numsubloci[world->locus],sizeof(MYREAL*));
     long sublocus;
@@ -703,6 +367,7 @@ set_new_proposal (proposal_fmt ** proposal, timelist_fmt * tl, world_fmt * world
 {
   //long mal = world->data->maxalleles[world->locus];
   //long oldsize=0;
+  error("needs fixing");
   long newsize =0;
   mutationmodel_fmt *s;
   long sumtips = world->sumtips;
@@ -734,8 +399,8 @@ set_new_proposal (proposal_fmt ** proposal, timelist_fmt * tl, world_fmt * world
     (*proposal)->aboveorigin = (*proposal)->nodedata;
     (*proposal)->bordernodes = (*proposal)->nodedata + listsize;
     (*proposal)->line_f =   (*proposal)->bordernodes + listsize;
+    (*proposal)->line_t_allocsize = listsize;
     (*proposal)->line_t =  (*proposal)->line_f + listsize;
-    (*proposal)->listsize =  listsize;
     // mf holds also mt array  
 
     (*proposal)->mf = (MYREAL **) calloc(world->numsubloci[world->locus]*2,sizeof(MYREAL*));
@@ -905,7 +570,7 @@ chooseOrigin (proposal_fmt * proposal)
     node *tmp=NULL, **goal;
     //032110 goal = (node **) mycalloc (oldelem, sizeof (node *));
     //#ifdef DEBUG
-    //traverse_check(crawlback (proposal->root->next));
+    traverse_check(crawlback (proposal->root->next));
     //#endif
     //032110 traverseAllNodes (crawlback (proposal->root->next), &goal, &elem, &oldelem,
     //032110                  NO_MIGR_NODES);
@@ -957,10 +622,11 @@ construct_localtimelist (timelist_fmt * timevector, proposal_fmt * proposal)
     long ii;
 #endif
     long z = 0;
-    long oz = proposal->listsize;
+    long oz = proposal->aboveorigin_allocsize;
     //    long numpop = timevector->numpop = proposal->numpop;
     traverseAllNodes (crawlback (proposal->origin)->back,
-                      &proposal->aboveorigin, &z, &oz, WITH_MIGR_NODES);
+                      &proposal->aboveorigin, &z, &oz, WITH_MIGR_NODES,&proposal->divlist);
+    proposal->aboveorigin_allocsize = oz;
     proposal->aboveorigin[z++] = proposal->oback;
     proposal->aboveorigin[z] = NULL;
     prune_timelist(&proposal->world->treetimes[0],timevector, proposal->aboveorigin, proposal, proposal->world->numpop);
@@ -975,99 +641,101 @@ construct_localtimelist (timelist_fmt * timevector, proposal_fmt * proposal)
         long jj;
         for(jj = 1;jj < proposal->numpop; jj++)
             printf(" %3li",timevector->tl[ii].lineages[jj]);
-        printf(" backtyme:%f dif:%f\n",showtop(timevector->tl[ii].eventnode->back)->tyme,dif=showtop(timevector->tl[ii].eventnode->back)->tyme-timevector->tl[ii].eventnode->tyme);
-        if(dif<0.0)
-        {
-            error("shit");
-        }
+        printf(" backtyme:%f dif:%f\n",showtop(showtop(timevector->tl[ii].eventnode)->back)->tyme,dif=showtop(timevector->tl[ii].eventnode->back)->tyme-timevector->tl[ii].eventnode->tyme);
     }
 #endif
 }
 
+void record_above_origin_divergence(div_fmt **divlist, long from, long to)
+{ 
+  if ( (*divlist)->div_elem >= (*divlist)->div_allocsize)
+    {
+      (*divlist)->div_allocsize += HUNDRED;
+      (*divlist)->divlist = (longpair *) realloc((*divlist)->divlist, sizeof(longpair)*(*divlist)->div_allocsize);
+    }
+  (*divlist)->divlist[(*divlist)->div_elem][0]=from;
+  (*divlist)->divlist[(*divlist)->div_elem][1]=to;
+  (*divlist)->div_elem += 1;
+}
+
+void allocate_nodelist(node ***nodelist, long *oldnode_elem, long elements)
+{
+  long olde = (*oldnode_elem);
+  (*oldnode_elem) += elements;
+  (*nodelist) = (node **) realloc(*nodelist, sizeof(node *) * (*oldnode_elem));
+  memset(*nodelist + olde, 0,sizeof(node*) * elements);
+}
+
+void test_traverse_error(node * theNode)
+{
+  if(theNode->type != 'r' && theNode->tyme > showtop(theNode->back)->tyme)
+    {
+      printf("%i> Tree traverse failed\nwith node (%c) time %10.10f and node (%c) time %10.10f (diff=%20.20f)\n",
+	     myID, theNode->type,  theNode->tyme, showtop(theNode->back)->type, showtop(theNode->back)->tyme,theNode->tyme - showtop(theNode->back)->tyme);
+      error("traverseAllNodes() failed");
+    }
+}
+
 /*----------------------------------------------------------------------------
  finds all nodes in a tree starting at the root node and crawling up 
  to the tips in a recursive fashion, writing nodeptrs in the nodelist vector
  the flag include_migration is 1 if we want to touch the migration nodes too,
  otherwise =0 -> jump over the migration nodes. for convenience we define the 
- the macros NO_MIGR_NODES=0 and WITH_MIGR_NODES=1 in the treesetup.h file
- PB 1995
+ the macros NO_MIGR_NODES=0 and WITH_MIGR_NODES=1 in the definitions.h file
  */
 void
 traverseAllNodes (node * theNode, node *** nodelist, long *node_elem,
-                  long *oldnode_elem, int include_migration)
+                  long *oldnode_elem, int include_migration, div_fmt **divlist)
 {
-    static long counter=0;
-    long elem;
-    counter++;
-    if(theNode->type != 'r' && theNode->tyme > showtop(theNode->back)->tyme)
-    {
-        printf("%i> TTRAVERSETRAVERSETRAVERSETRAVERSE\nfunction was called with %li times: with node (%c) time %10.10f and node (%c) time %10.10f (diff=%20.20f)\n",
-               myID, counter, theNode->type,  theNode->tyme, showtop(theNode->back)->type, showtop(theNode->back)->tyme,theNode->tyme - showtop(theNode->back)->tyme);
-        error("traverseAllNodes() failed");
-    }
-    
+    test_traverse_error(theNode);
     if (include_migration == NO_MIGR_NODES)
     {
-        // nodelist needs to be at least twice as long as sumtips
-        // otherwise it will run out of reserved memory 
         if (theNode->type != 't')
         {
-            if (crawlback (theNode->next) != NULL)
-                traverseAllNodes (crawlback (theNode->next), nodelist, node_elem,
-                                  oldnode_elem, NO_MIGR_NODES);
-            if (theNode->type != 'm' && theNode->type != 'd' && crawlback (theNode->next->next) != NULL)
-                traverseAllNodes (crawlback (theNode->next->next), nodelist,
-                                  node_elem, oldnode_elem, NO_MIGR_NODES);
-            (*nodelist)[(*node_elem)] = theNode;
-            (*node_elem) += 1;
-            if (theNode->type == 'm' || theNode->type == 'd')
-            {
-                error ("Migration/Speciaton node encountered?! and died!");
-            }
-            if(*node_elem > *oldnode_elem)
-                error("die on the spot, this should not happen. Failed in traverseAllNodes without migration nodes");
+	  node *testnode = crawlback(theNode->next);
+	  if (testnode != NULL)
+	    traverseAllNodes (testnode, nodelist, node_elem, oldnode_elem, NO_MIGR_NODES, divlist);
+	  node *testnode2 = crawlback (theNode->next->next);
+	  if (testnode2 != NULL && theNode->type != 'm' && theNode->type != 'd')
+	    traverseAllNodes (testnode2, nodelist, node_elem, oldnode_elem, NO_MIGR_NODES, divlist);
+	  if (*node_elem >= *oldnode_elem-3)
+	    allocate_nodelist(nodelist,oldnode_elem, HUNDRED);
+	  (*nodelist)[(*node_elem)] = theNode;
+	  (*node_elem) += 1;
         }
         else
         {
-            (*nodelist)[(*node_elem)] = theNode;
-            (*node_elem) += 1;
-            if(*node_elem > *oldnode_elem)
-                error("die on the spot, this should not happen. Failed in traverseAllNodes without migration nodes at tip");
+	  if (*node_elem >= *oldnode_elem-3)
+	    allocate_nodelist(nodelist,oldnode_elem, HUNDRED);
+	  (*nodelist)[(*node_elem)] = theNode;
+	  (*node_elem) += 1;
         }
     }
     else
     {
         if (theNode->type != 't')
         {
-            if (theNode->next->back != NULL)
-                traverseAllNodes (theNode->next->back, nodelist, node_elem,
-                                  oldnode_elem, WITH_MIGR_NODES);
-            if (theNode->type != 'm' && theNode->type != 'd' && theNode->next->next->back != NULL)
-                traverseAllNodes (theNode->next->next->back, nodelist, node_elem,
-                                  oldnode_elem, WITH_MIGR_NODES);
-            if ((*node_elem) == (*oldnode_elem - 2))
-            {
-                elem = (*oldnode_elem) + (*oldnode_elem);
-                (*nodelist) =
-                (node **) myrealloc ((*nodelist), sizeof (node *) * elem);
-                memset ((*nodelist) + (*oldnode_elem), 0,
-                        sizeof (node *) * (elem - (*oldnode_elem)));
-                *oldnode_elem = elem;
-            }
-            (*nodelist)[(*node_elem)++] = theNode;
+	  if (!showtop(theNode))
+	    {
+	      error("not topnode");
+	    }
+	  if (theNode->next->back != NULL)
+	    traverseAllNodes (theNode->next->back, nodelist, node_elem,
+			      oldnode_elem, WITH_MIGR_NODES, divlist);
+	  if (divlist != NULL && theNode->type == 'd')
+	    record_above_origin_divergence(divlist, theNode->pop,theNode->actualpop);
+	  if (theNode->type == 'i' && theNode->next->next->back != NULL)
+	    traverseAllNodes (theNode->next->next->back, nodelist, node_elem,
+			      oldnode_elem, WITH_MIGR_NODES, divlist);
+	  if (*node_elem >= *oldnode_elem-3)
+	    allocate_nodelist(nodelist,oldnode_elem, HUNDRED);
+	  (*nodelist)[(*node_elem)++] = theNode;
         }
         else
         {
-            if ((*node_elem) == (*oldnode_elem - 2))
-            {
-                elem = (*oldnode_elem) + (*oldnode_elem);
-                (*nodelist) =
-                (node **) myrealloc ((*nodelist), sizeof (node *) * elem);
-                memset ((*nodelist) + (*oldnode_elem), 0,
-                        sizeof (node *) * (elem - (*oldnode_elem)));
-                *oldnode_elem = elem;
-            }
-            (*nodelist)[(*node_elem)++] = theNode;
+	  if (*node_elem >= *oldnode_elem-3)
+	    allocate_nodelist(nodelist,oldnode_elem, HUNDRED);
+	  (*nodelist)[(*node_elem)++] = theNode;
         }
     }
 }
@@ -1105,6 +773,7 @@ prune_timelist (timelist_fmt * oldtv, timelist_fmt * newtv,  register node ** __
 	      tls->interval  = (*oldtv).tl[i].interval ;
 	      if(thenode!=NULL)
 		{
+
 		  tls->from      = thenode->pop;
 		  tls->to        = thenode->actualpop;
 		}
@@ -1171,13 +840,26 @@ migrate (proposal_fmt * proposal, node * up, char event)
     else
         array[i].to = up->pop;
     if(event=='d')
-      	tmp = speciation_from (array[i].to, proposal);
+      {
+     	tmp = speciation_from (array[i].to, proposal);
+	if (tmp == -1)
+	  return 0;
+      }
     else
       {
 	tmp = migration_from (array[i].to, proposal);
 	event = 'm';
+	if (tmp == -1)
+	  return 0;
       }
     array[i].event = event;
+    //#if DEBUG
+    //if (i>0)
+    //  {
+    //	if (array[i-1].event == 'd' && event == 'm')
+    //	  error("GOPF");
+    //  }
+    //#endif
     if(tmp < numpop)
         array[i].from = tmp;
     else
@@ -1219,11 +901,17 @@ migrateb (proposal_fmt * proposal, node * up, char event)
         array[i].to = up->pop;
 
     if(event=='d')
+      {
       	tmp = speciation_from (array[i].to, proposal);
+	if (tmp == -1)
+	  return 0;
+      }
     else
       {
 	tmp = migration_from (array[i].to, proposal);
 	event = 'm';
+	if (tmp == -1)
+	  return 0;
       }
     array[i].event = event;
     if(tmp < numpop)
@@ -1282,7 +970,9 @@ chooseTarget (proposal_fmt * proposal, timelist_fmt * timevector,
     {
         error ("Wrong Time for action in chooseTarget()\n");
     }
-    findbordernodes (rb, proposal, actualpop, &bordernodes, &proposal->listsize, bordernum,
+    //printf("\n\n\n");
+    findbordernodes (rb, proposal, actualpop, &bordernodes, 
+		     &proposal->bordernodes_allocsize, bordernum,
                      &(*timevector).tl, (*timevector).T);
     if (*bordernum > 0)
     {
@@ -1328,17 +1018,25 @@ findbordernodes (node * theNode, proposal_fmt * proposal, long pop,
         tmp = showtop (theNode);
         back = showtop (theNode->back);
     }
+    //printf("%i> fb: %p %f <? %f <? %f (%c%c)[back%li tmp%li]",myID,proposal, back->tyme,proposal->time,tmp->tyme,back->type, tmp->type, back->actualpop,tmp->pop);  
     if (pop == tmp->pop && pop == back->actualpop && tmp->tyme < proposal->time
         && back->tyme > proposal->time)
     {
         // lineage end points bracket the the new proposed time,
         // therefore this branch needs to be included
-        (*bordernodes)[(*bordernum)++] = tmp;
-        return;
+      if (*bordernum >= *allocsize-3)
+	{
+	  allocate_nodelist(bordernodes, allocsize, HUNDRED);      
+	}
+      (*bordernodes)[*bordernum] = tmp;
+      *bordernum += 1;
+      //printf("****\n");
+      return;
     }
     else
     {
-        // the lineage does not fit the populations (it may fit the time, so)
+      //printf("\n");
+      // the lineage does not fit the populations (it may fit the time, so)
         // the if here checks this
         if (back->tyme < proposal->time)
         {
@@ -1355,14 +1053,49 @@ findbordernodes (node * theNode, proposal_fmt * proposal, long pop,
         // we encounter a tip node (this is important with dated tips because these can
         // be interspersed in the timelist.
         if (tmp->type != 't')
-        {
-            if (tmp->next->back != NULL)
-                findbordernodes (tmp->next->back, proposal, pop, bordernodes, allocsize,
-                                 bordernum, tyme, gte);
-            if (tmp->type != 'm' && tmp->type != 'd' && tmp->next->next->back != NULL)
-                findbordernodes (tmp->next->next->back, proposal, pop,
-                                 bordernodes, allocsize, bordernum, tyme, gte);
-        }
+	  {
+	    if (tmp->next->back != NULL)
+	      {
+		//node *tmpup = crawlback(tmp->next);
+		//if (tmpup->tyme > proposal->time && back->tyme > proposal->time)
+		//  {
+		//  if(tmpup->type == 'i')
+		//    {
+		//findbordernodes (tmpup->next->back, proposal, pop, bordernodes, allocsize,
+		//		 bordernum, tyme, gte);
+		//findbordernodes (tmpup->next->next->back, proposal, pop,
+		//		 bordernodes, allocsize, bordernum, tyme, gte);
+		//    }
+		//}
+		//else
+		//{
+		    findbordernodes (tmp->next->back, proposal, pop, bordernodes, allocsize,
+				     bordernum, tyme, gte);
+		    //}
+	      }
+	    if (tmp->type == 'i' && tmp->next->next->back != NULL)
+	      findbordernodes (tmp->next->next->back, proposal, pop,
+			       bordernodes, allocsize, bordernum, tyme, gte);
+	  }
+#ifdef DEBUGXXXX
+	else
+	  {
+	    if (pop == tmp->pop && pop == back->actualpop && tmp->tyme < proposal->time
+		&& back->tyme > proposal->time)
+	      {
+		// lineage end points bracket the the new proposed time,
+		// therefore this branch needs to be included
+		if (*bordernum >= *allocsize-3)
+		  {
+		    allocate_nodelist(bordernodes, allocsize, HUNDRED);      
+		  }
+		(*bordernodes)[*bordernum] = tmp;
+		*bordernum += 1;
+		//printf("***t\n");
+		return;
+	      }
+	  }
+#endif
     }
 }
 
@@ -1653,11 +1386,9 @@ reset_proposal (proposal_fmt ** proposal, world_fmt *world)
 void
 free_masterproposal (proposal_fmt * proposal)
 {
-#ifndef TESTING2
-  free_masterproposal(proposal);
-#endif  
+  //implement  free_masterproposal(proposal);
 }
-#endif /*TESTING2*/
+#else
 void
 free_masterproposal (proposal_fmt * proposal)
 {
@@ -1669,11 +1400,14 @@ free_masterproposal (proposal_fmt * proposal)
   
   //static long count=0;
   myfree(proposal->aboveorigin);
-  // myfree(proposal->bordernodes);
-  // myfree(proposal->line_f);
-  // myfree(proposal->line_t);
+  myfree(proposal->bordernodes);
+  myfree(proposal->line_f);
+  myfree(proposal->line_t);
+  myfree(proposal->divlist->divlist);
+  myfree(proposal->divlist);
+  myfree(proposal->param0save);
   //  printf("%li ",count++);fflush(stdout);
-  myfree(proposal->nodedata); // ..line_t are also freed with this
+  //myfree(proposal->nodedata); // ..line_t are also freed with this
     for(sublocus=sublocistart; sublocus < sublociend; sublocus++)
       {
 	s = &proposal->world->mutationmodels[sublocus];
@@ -1716,18 +1450,16 @@ free_masterproposal (proposal_fmt * proposal)
     myfree(proposal);
 }
 
+#endif  /*TESTING2*/
+
 
 void
 free_timevector (timelist_fmt * timevector)
 {
-    //    long i;
-    //    for (i = 0; i < timevector->allocT; i++)
-    //    {
-    //        myfree(timevector->tl[i].lineages);
-    //    }
     myfree(timevector->lineages);
     myfree(timevector->tl);
     myfree(timevector);
+    //printf("%i> free timevector\n",myID);
 }
 
 ///
@@ -1751,7 +1483,7 @@ acceptlike (world_fmt * world, proposal_fmt * proposal, long g,
 {
     //proposal changes when chain is stuck, to accept some new trees without
     //considering data to remove out of sticky area [hopefully]
-    static long not_accepted = 0;
+    //static long not_accepted = 0;
     
     const long limit = MIGRATION_LIMIT * world->numpop;
     
@@ -1821,7 +1553,7 @@ acceptlike (world_fmt * world, proposal_fmt * proposal, long g,
     // }
     if (world->likelihood[g] < proposal->likelihood)
     {
-        not_accepted = 0;
+      //not_accepted = 0;
         return TRUE;
     }
     if (!world->options->heating)
@@ -1830,7 +1562,7 @@ acceptlike (world_fmt * world, proposal_fmt * proposal, long g,
         rr = LOG (RANDUM ());
         if (rr < expo)
         {
-            not_accepted = 0;
+	  //not_accepted = 0;
             return TRUE;
         }
     }
@@ -1840,7 +1572,7 @@ acceptlike (world_fmt * world, proposal_fmt * proposal, long g,
         rr = LOG (RANDUM ());
         if (rr < expo)
         {
-            not_accepted = 0;
+	  //not_accepted = 0;
             return TRUE;
         }
     }
