@@ -91,7 +91,7 @@ void t_s_tcon (proposal_fmt * proposal, node * uppern);
 long erase_migr_nodes (world_fmt *world, node * up);
 node *findcrossing (node ** ptrl1, node ** ptrl2);
 void connectnodes (world_fmt *world, node * mother, node * brother, node * sister);
-void gotoroot (node * origin, node ** ptrlist);
+void gotoroot (node * origin, node ** ptrlist, long *ptrallocsize);
 void adjust (node * theNode, MYREAL tyme, long level);
 void localevaluate (node * mother);
 void copy_x (proposal_fmt * proposal, xarray_fmt *axx1, xarray_fmt *axx2);
@@ -220,6 +220,7 @@ coalesce1p (proposal_fmt * proposal)
     fix_root_pop (proposal->world, crawlback (proposal->root->next));
     //    if(proposal->world->in_last_chain)
     traverse_tagnew(crawlback (proposal->root->next),proposal->origin);
+    traverse_check(crawlback (proposal->root->next));
 #ifdef MESS
     //traverse_checker(proposal->root);
     //if(skip==0)
@@ -786,7 +787,7 @@ void target_stancoa (proposal_fmt * proposal)
     if (proposal->ocousin != NULL)
       {
         obb = showtop (crawlback (proposal->oback));
-        gotoroot (proposal->target, proposal->line_t); // looks at target --> .... ---> root
+        gotoroot (proposal->target, proposal->line_t, &proposal->line_t_allocsize); // looks at target --> .... ---> root
         double_line[0] = proposal->osister;          
         /*findcrossing needs a last NULL element */
         if (findcrossing (double_line, proposal->line_t) != NULL)
@@ -811,7 +812,7 @@ void target_stancoa (proposal_fmt * proposal)
 	      }
             else
 	      {
-                gotoroot (obb, proposal->line_f); //goto the root from obb, the node that is below origin and stable
+                gotoroot (obb, proposal->line_f, &proposal->line_f_allocsize); //goto the root from obb, the node that is below origin and stable
                 proposal->connect = findcrossing (proposal->line_f, proposal->line_t); // node that is ancestor of obb and target
                 proposal->vs = obb->tyme - proposal->osister->tyme;
                 proposal->v = obb->tyme - proposal->ocousin->tyme;
@@ -965,7 +966,7 @@ void old_target_stancoa (proposal_fmt * proposal)
     if (proposal->ocousin != NULL)
       {
         obb = showtop (crawlback (proposal->oback));
-        gotoroot (proposal->target, proposal->line_t);
+        gotoroot (proposal->target, proposal->line_t, &proposal->line_t_allocsize);
         double_line[0] = proposal->osister;
         /*findcrossing needs a last NULL element */
         if (findcrossing (double_line, proposal->line_t) != NULL)
@@ -982,7 +983,7 @@ void old_target_stancoa (proposal_fmt * proposal)
 	      }
             else
 	      {
-                gotoroot (obb, proposal->line_f);
+                gotoroot (obb, proposal->line_f, &proposal->line_f_allocsize);
                 proposal->connect =
 		  findcrossing (proposal->line_f, proposal->line_t);
                 proposal->vs = obb->tyme - proposal->osister->tyme;
@@ -1700,7 +1701,7 @@ connectnodes (world_fmt *world, node * mother, node * brother, node * sister)
 }
 
 void
-gotoroot (node * origin, node ** ptrlist)
+gotoroot (node * origin, node ** ptrlist, long *ptrallocsize)
 {
     node *theNode;
     long i = 0;
@@ -1709,7 +1710,9 @@ gotoroot (node * origin, node ** ptrlist)
             (theNode != NULL) && (crawlback (theNode)->type != 'r');
             theNode = showtop (crawlback (theNode)))
     {
-        ptrlist[i++] = theNode;
+      if (i >= *ptrallocsize-3)
+	allocate_nodelist(&ptrlist,ptrallocsize, HUNDRED);
+      ptrlist[i++] = theNode;
     }
     ptrlist[i] = theNode;  /* adds root->back to the list */
     ptrlist[i+1] = NULL;   /* guarantees an end for the array)*/
