@@ -419,11 +419,12 @@ typedef struct _haplocount
   long count;
 } haplocount_fmt;  
 
-/// define the holding structure haplotype individual names in the data opbject 
-typedef struct _individualHOLD
-{
-  
-} individualHOLD;
+// ready to delete ? May 11 2015
+/// define the holding structure haplotype individual names in the data object 
+//typedef struct _individualHOLD
+//{
+//  
+//} individualHOLD;
 
 /// define the structure to hold the individual name and haplotype information
 typedef struct _individualDB 
@@ -548,9 +549,9 @@ typedef struct _bayeshistogram
     MYREAL *maxima;    // contains maximal values for each parameter
     MYREAL *adjmaxima;// holds maxima values used in histogram [are smaller than maxima]
     MYREAL *cred50l;    // holds 50%-credibility margins (<all lower values>, 
-    MYREAL *cred50u;   //<all high values>)
+    MYREAL *cred50u;   // <all high values>)
     MYREAL *cred95l;    // holds 95%-credibility margins (<all lower values>)
-    MYREAL *cred95u;   //<all high values>)
+    MYREAL *cred95u;   // <all high values>)
     MYREAL *modes;    // holds 95%-credibility margins (<all lower values>, <all high values>)
     MYREAL *medians;
     MYREAL *means;
@@ -559,10 +560,27 @@ typedef struct _bayeshistogram
   long n; //holds the number of samples taken, this seem to be missing for the reading from mdimfile, but available otherwise
 } bayeshistogram_fmt;
 
+typedef struct _hyper_fmt
+{
+  MYREAL alpha;
+  MYREAL alphastd;
+  long alphan;
+  MYREAL mean;
+  MYREAL meanstd;
+  long meann;
+  long count;
+} hyper_fmt; 
+
 /// holds data for Bayesian approach
 typedef struct _bayes
 {
   long count;
+  long hypercount;
+  long hyperinterval;
+  boolean hyperprior;
+  MYREAL hyperfactormean;
+  MYREAL hyperfactoralpha;
+  hyper_fmt * hyper;
   long numpop2;
   long mapsize;
   longpair *map; //mapping of parameter using custom migration matrix
@@ -576,6 +594,7 @@ typedef struct _bayes
   MYREAL *maxparam; // maximal allowed parameter value
   MYREAL *meanparam; // mean values for expprior
   MYREAL *alphaparam; // alpha value for gamma
+  MYREAL *alphaorigparam; // alpha value for gamma (original [used for Hyperp])
   MYREAL *betaparam; // beta value for gamma
   // record all changes of parameters
   MYREAL *params;  // save for parameter vectors
@@ -756,7 +775,8 @@ typedef struct _option
   boolean mdimdelete;
   int use_compressed;
   boolean prioralone;
-
+  FILE *divtimefile;
+  char *divtimefilename;
   char title[LINESIZE + 1];
   lratio_fmt *lratio;
   boolean aic;
@@ -899,6 +919,7 @@ typedef struct _option
   long heated_chains;
   /* save genealogy summary options */
   boolean readsum;
+  boolean checkpointing;
   boolean writesum;
   /* threading over loci */
   int cpu;
@@ -933,6 +954,12 @@ typedef struct _option
   boolean has_speciation;
   boolean tersepdf; // prints only summaries and not per locus information
   boolean allposteriors; // plot all loci-posteriors
+  long **unfinished;
+  boolean hyperprior;
+  long hyperinterval;
+  MYREAL hyperfactormean;
+  MYREAL hyperfactoralpha;
+  boolean recorddivtime;
 }
 option_fmt;
 
@@ -1326,6 +1353,14 @@ typedef struct _mpirequest_fmt
 /// holds all the parameters etc to run the program
 
 //typedef struct proposal_fmt;
+typedef struct _divtime_fmt 
+{
+  double age;
+  long from;
+  long to;
+} divtime_fmt; 
+
+
 
 typedef struct _world
 {
@@ -1553,6 +1588,10 @@ typedef struct _world
   boolean has_speciation;
   species_fmt *species_model;
   long species_model_size;
+  divtime_fmt *divtime;
+  long divtime_alloc;
+  long divtime_num;
+  FILE *divtimefile;
 }
 world_fmt;
 
@@ -1642,70 +1681,83 @@ typedef struct _migr_table_fmt
 }
 migr_table_fmt;
 
+typedef struct _div_fmt 
+{
+  longpair *divlist;
+  long div_allocsize;
+  long div_elem;
+} div_fmt;
+
+
 ///
 /// during the treeupdates we need a scratchpad, proposal_fmt
 /// holds all temporary data
 typedef  struct _proposal_fmt
 {
-    world_fmt *world;
-    char datatype;
-    long sumtips;
-    long numpop;
-    long endsite;
-    MYREAL fracchange;
-    MYREAL *param0;
-    node *root;
-    short migration_model;
-    long listsize; // holds the size of several nodelists
-    boolean mig_removed;
-    MYREAL rr;
-    node **nodedata;      // holds abovenodes and bordernodes
-    node *origin;            // pointers into nodedata
-    node *target;            // .... 
-    node *realtarget;
-    node *tsister;
-    node *realtsister;
-    node *osister;
-    node *realosister;
-    node *ocousin;
-    node *realocousin;
-    node *oback;
-    node *realoback;
-    node **line_f;
-    node **line_t;
-    node *connect;
-    MYREAL likelihood;
-    MYREAL ueplikelihood;
-    MYREAL **ueplike;
-    MYREAL time;
-    MYREAL v;
-    MYREAL vs;
-    xarray_fmt *xt;
-    xarray_fmt *xf;
-    MYREAL **mf;
-    MYREAL **mt;
-//could be used to calculate p(gn|go):   MYREAL nu;
+  world_fmt *world;
+  char datatype;
+  long sumtips;
+  long numpop;
+  long endsite;
+  MYREAL fracchange;
+  MYREAL *param0;
+  MYREAL *param0save;
+  node *root;
+  short migration_model;
+  boolean mig_removed;
+  MYREAL rr;
+  node **nodedata;      // holds abovenodes and bordernodes
+  node *origin;            // pointers into nodedata
+  node *target;            // .... 
+  node *realtarget;
+  node *tsister;
+  node *realtsister;
+  node *osister;
+  node *realosister;
+  node *ocousin;
+  node *realocousin;
+  node *oback;
+  node *realoback;
+  long line_f_allocsize;
+  long line_t_allocsize;
+  node **line_f;
+  node **line_t;
+  node *connect;
+  MYREAL likelihood;
+  MYREAL ueplikelihood;
+  MYREAL **ueplike;
+  MYREAL time;
+  MYREAL v;
+  MYREAL vs;
+  xarray_fmt *xt;
+  xarray_fmt *xf;
+  MYREAL **mf;
+  MYREAL **mt;
+  //could be used to calculate p(gn|go):   MYREAL nu;
 #ifdef UEP
-    ueparray_fmt ut;
-    ueparray_fmt uf;
-    node *firstuep;
-    MYREAL *umf;
-    MYREAL *umt;
+  ueparray_fmt ut;
+  ueparray_fmt uf;
+  node *firstuep;
+  MYREAL *umf;
+  MYREAL *umt;
 #endif
-
-    node **aboveorigin;
-    node **bordernodes;
-    migr_table_fmt *migr_table;
-    migr_table_fmt *migr_table2;
-    long migr_table_counter;
-    long migr_table_counter2;
-    long old_migr_table_counter;
-    long old_migr_table_counter2;
-    long timeslice;
-    MYREAL *mig0list;
-    long *design0list;
-    MYREAL treelen;
-
+  long aboveorigin_allocsize;
+  node **aboveorigin;
+  long divlist_allocsize;
+  div_fmt *divlist;
+  long bordernodes_allocsize;
+  node **bordernodes;
+  migr_table_fmt *migr_table;
+  migr_table_fmt *migr_table2;
+  long migr_table_counter;
+  long migr_table_counter2;
+  long old_migr_table_counter;
+  long old_migr_table_counter2;
+  long timeslice;
+  MYREAL *mig0list;
+  long *design0list;
+  MYREAL treelen;
+  
 #ifdef BEAGLE
   long parentid;
   long leftid;
