@@ -1164,6 +1164,51 @@ void finish_mutationmodel(world_fmt *world, data_fmt *data, option_fmt *options,
     }
 }
 
+void destroy_mutationmodel(world_fmt* world)
+{
+  long sublocus;
+  long locus;
+  long i,j;
+  for(locus=0;locus<world->loci;locus++)
+    {
+      long   sublocistart = world->sublocistarts[locus];
+      long   sublociend = world->sublocistarts[locus+1];
+      mutationmodel_fmt *s;      
+      for(sublocus=sublocistart;sublocus<sublociend;sublocus++)
+	{
+	  s =&world->mutationmodels[sublocus];
+	  myfree(s->parameters);
+	  myfree(s->basefreqs);
+	  myfree(s->siterates);
+	  myfree(s->siteprobs);
+	  myfree(s->rate);
+	  myfree(s->eigenvectormatrix);
+	  myfree(s->inverseeigenvectormatrix);
+	  myfree(s->eigenvalues);
+	  myfree(s->alias);
+	  myfree(s->ally);
+	  myfree(s->weight);
+	  myfree(s->aliasweight);
+	  myfree(s->location);
+	  myfree(s->savealiasweight);
+	  myfree(s->category);
+	  myfree(s->contribution);
+	  if(s->tbl!=NULL)
+	    {
+	      for(i = 0 ; i < s->numsiterates; i++)
+		{
+		  for (j = 0; j < world->options->categs; j++)	      
+		    myfree(s->tbl[i][j]);
+		}
+	    }
+	}
+      if(s->tbl!=NULL)
+	myfree(s->tbl);
+    }
+  myfree(world->mutationmodels);
+}
+
+
 void print_mutationrate_weights(FILE *file, MYREAL *murates, long *segregs, MYREAL *wattersons, long loci)
 {
   long locus;
