@@ -50,7 +50,7 @@ extern int myID;
 
 MYREAL cdf_gamma(MYREAL a, MYREAL b, MYREAL x);
 MYREAL trunc_gamma_rand(MYREAL alpha, MYREAL beta, MYREAL lower, MYREAL upper);
-
+MYREAL normal_rand(MYREAL mean, MYREAL std);
 ///
 /// Normal distribution
 /// this generates only one random number, but could be easily changed to supply two numbers
@@ -347,6 +347,82 @@ MYREAL find_beta_truncgamma(MYREAL mean, MYREAL alpha, MYREAL lower, MYREAL uppe
   return beta;
 }
 	
+void init_hyperpriorrecord(hyper_fmt ** hyper,long numparam)
+{
+  long i;
+  (*hyper) = (hyper_fmt *) calloc(numparam, sizeof(hyper_fmt));
+  for(i=0;i<numparam;i++)
+    {
+      onepass_mean_std_start(&(*hyper)[i].alpha, &(*hyper)[i].alphastd, &(*hyper)[i].alphan);
+      onepass_mean_std_start(&(*hyper)[i].mean, &(*hyper)[i].meanstd, &(*hyper)[i].meann);
+    }
+}
+
+void hyper_normal(MYREAL *mean, MYREAL *std, MYREAL origmean, MYREAL origstd, MYREAL lower, MYREAL upper, MYREAL factor)
+{
+  // use a normal to draw new means, alpha, and then recalculate beta
+  // using the original prior mean as the mean and std of the normal
+  MYREAL nmean = normal_rand(origmean,factor*origmean);
+  MYREAL nstd = normal_rand(origstd,factor*origstd);
+  while (nstd <= EPSILON)
+    {
+      nstd = normal_rand(origstd,origstd);
+    }
+  while ((nmean <= lower) || (nmean > upper))
+    {
+      nmean = normal_rand(origmean,origmean);
+    }
+  *mean = nmean;
+  *std = nstd;
+}
+void hyper_gamma(MYREAL *alpha, MYREAL *beta, MYREAL origmean, MYREAL origalpha, MYREAL lower, MYREAL upper, MYREAL factormean, MYREAL factoralpha)
+{
+  // use a normal to draw new means, alpha, and then recalculate beta
+  // using the original prior mean as the mean and std of the normal
+  MYREAL nmean = normal_rand(origmean,factormean*origmean);
+  MYREAL nalpha = origalpha;
+  if(factoralpha>0.0)
+    {
+      nalpha = normal_rand(origalpha,factoralpha*origalpha);
+      while (nalpha < lower || nalpha > upper)
+	{
+	  nalpha = normal_rand(origalpha, factoralpha*origalpha);
+	}
+    }
+  while (nmean < lower || nmean > upper)
+    {
+      nmean = normal_rand(origmean,factormean*origmean);     
+    }
+  MYREAL nbeta = find_beta_truncgamma(nmean,nalpha,lower,upper);
+  //printf("%i> %c nmean=%f origmean=%f alpha=%f, beta=%f\n", myID, nmean < origmean ? '-' : '+', nmean, origmean, nalpha, nbeta);
+  *alpha = nalpha;
+  *beta = nbeta;
+
+}
+
+void hyper_gamma_record(MYREAL alpha, MYREAL beta, hyper_fmt * hg)
+{
+  onepass_mean_std_calc(&hg->alpha,&hg->alphastd,&hg->alphan,alpha);
+  onepass_mean_std_calc(&hg->mean,&hg->meanstd,&hg->meann,alpha*beta);
+}
+
+void hyper_report(world_fmt *world, bayes_fmt *bayes)
+{
+  hyper_fmt *hyper = bayes->hyper;
+  long i;
+  fprintf(world->outfile, "Hyperpriors are active\n");
+  fprintf(world->outfile, "----------------------\n");
+  fprintf(world->outfile, "Hyper prior is distributed: Normal(priormean,%f*std)\n",bayes->hyperfactormean);  
+  fprintf(world->outfile, "                            Normal(prioralpha,%f*std)\n",bayes->hyperfactoralpha);
+  fprintf(world->outfile, "Parameter     PriorMean Std  PriorAlpha   Std     N\n");
+  for (i=0;i<bayes->numparams;i++)
+    {
+      fprintf(world->outfile,"%li        %f  %f  %f %f %li\n", 
+	      i+1, hyper->mean, hyper->meanstd, 
+	      hyper->alpha, hyper->alphastd, 
+	      hyper->alphan);
+    }
+}
 
 ///
 /// Gamma prior retrieve a new value from a truncated gamma between lower and upper
@@ -362,17 +438,23 @@ propose_gamma_newparam (MYREAL param, long which, bayes_fmt * bayes, MYREAL *r)
   // const MYREAL lower = l < bayes->minparam[which] ? bayes->minparam[which] : l;
   // const MYREAL upper = u > bayes->maxparam[which] ? bayes->maxparam[which] : u;
   const MYREAL lower = bayes->minparam[which];
-  const MYREAL upper = bayes->maxparam[which];
-  //const MYREAL mean = bayes->meanparam[which];
+  const MYREAL upper = bayes->maxparam[which]; 
   MYREAL alpha = bayes->alphaparam[which];
   MYREAL beta = bayes->betaparam[which];
+  if (bayes->hyperprior && bayes->hyper[which].count++ % bayes->hyperinterval == 0)
+    {
+      const MYREAL mean = bayes->meanparam[which];
+      hyper_gamma(&alpha, &beta, mean, bayes->alphaorigparam[which], lower, upper,bayes->hyperfactormean,bayes->hyperfactoralpha);
+      bayes->alphaparam[which] = alpha;
+      bayes->betaparam[which] = beta;
+      hyper_gamma_record(alpha,beta, &bayes->hyper[which]);
+    }
   //rr = trunc_gamma_rand(alpha, beta, param-delta, param+delta);
   rr = trunc_gamma_rand(alpha, beta, lower, upper);
   return rr;
 }
 
 
-
 ///
 /// Hastings ratio calculator for gamma distribution
 /// P(new -> old)    P(old)
