@@ -67,7 +67,7 @@ $Id: random.c 2169 2013-08-24 19:02:04Z beerli $
 #ifdef GRANDCENTRAL
 #include <dispatch/dispatch.h>
 #include <dispatch/semaphore.h>
-dispatch_semaphore_t semaphore;
+//extern dispatch_semaphore_t semaphore;
 #endif
 
 
@@ -126,7 +126,7 @@ MYINLINE MYREAL get_cud()
   //rr=kk*log(ps[jj]);
 #ifdef PTHREADS
 
-    if ((pthread_mutex_lock (&(heating_pool->random_lock))) != 0)
+  if (heating_pool!=NULL && (pthread_mutex_lock (&(heating_pool->random_lock))) != 0)
         error ("pthread_mutex_lock failed in get_cud()");
 #endif
 
@@ -148,7 +148,7 @@ MYINLINE MYREAL get_cud()
   value = rr - floor(rr);
 #ifdef PTHREADS
 
-    if ((pthread_mutex_unlock (&(heating_pool->random_lock))) != 0)
+    if (heating_pool!=NULL && (pthread_mutex_unlock (&(heating_pool->random_lock))) != 0)
         error ("pthread_mutex_unlock failed in get_cud()");
 #endif
 
@@ -503,7 +503,7 @@ void set_seed(long autoseed, unsigned long * inseed)
             break;
         }
 #endif
-      *inseed = (unsigned long) abs((long)timeseed);
+      *inseed = (unsigned long) labs((long)timeseed);
       break;
     case NOAUTO:
     case NOAUTOSELF:
@@ -513,7 +513,7 @@ void set_seed(long autoseed, unsigned long * inseed)
         break;
     }
 #ifdef GRANDCENTRAL
-   //    semaphore = dispatch_semaphore_create(0);
+   //   semaphore = dispatch_semaphore_create(1);
 #endif
 }
 
@@ -763,7 +763,7 @@ randum_thread (void)
 /* thread save random number generator */
 {
     MYREAL value;
-    if ((pthread_mutex_lock (&(heating_pool->random_lock))) != 0)
+    if (heating_pool!=NULL && (pthread_mutex_lock (&(heating_pool->random_lock))) != 0)
         error ("pthread_mutex_lock failed in random_thread()");
 #ifdef MERSENNE_TWISTER
     value = sfmt_genrand_res53(sfmtp);
@@ -781,7 +781,7 @@ randum_thread (void)
     seed[2] &= 1023;
     value = (((seed[0] / 2048.0 + seed[1]) / 2048.0 + seed[2]) / 1024.0);
 #endif
-    if ((pthread_mutex_unlock (&(heating_pool->random_lock))) != 0)
+    if (heating_pool!=NULL && (pthread_mutex_unlock (&(heating_pool->random_lock))) != 0)
         error ("pthread_mutex_unlock failed in randum_thread()");
     return value;
 }    /* threaded randum */
