@@ -477,7 +477,8 @@ void memdebug_free(void *ptr, const char file[], const long line)
     (memt_finish.tv_usec - memt_start.tv_usec) / 1.e6;
   oldsize = malloc_size(ptr);
   totalsize -= oldsize;
-  fprintf(memfile,"%i> %p %.9f . free (%li_%li) in %s at %li\n",myID, ptr, memelapsed, oldsize,totalsize, file, line);
+  fprintf(memfile,"%li %li %li %s\n",(long) oldsize, totalsize, line, file);
+  //  fprintf(memfile,"%i> %p %.9f . free (%li_%li) in %s at %li\n",myID, ptr, memelapsed, oldsize,totalsize, file, line);
   free(ptr);
 }
 #endif
@@ -511,7 +512,8 @@ LAMARC_malloc (size_t size, const char file[], const long line)
   memelapsed = memt_finish.tv_sec - memt_start.tv_sec +
     (memt_finish.tv_usec - memt_start.tv_usec) / 1.e6;
   totalsize += malloc_size(x);
-  fprintf(memfile,"%i> %p %.9f . malloc size=%li_%li in %s at %li\n",myID, x, memelapsed, (long) size, totalsize, file, line);
+  fprintf(memfile,"%li %li %li %s\n",(long) newsize, totalsize, line, file);
+  //  fprintf(memfile,"%i> %p %.9f . malloc size=%li_%li in %s at %li\n",myID, x, memelapsed, (long) size, totalsize, file, line);
 #endif    
     if (x == NULL)
     {
@@ -552,7 +554,8 @@ LAMARC_calloc (size_t repeats, size_t size,
   memelapsed = memt_finish.tv_sec - memt_start.tv_sec +
     (memt_finish.tv_usec - memt_start.tv_usec) / 1.e6;
   totalsize += malloc_size(x);
-    fprintf(memfile,"%i> %p %.9f . calloc:%lix%li  size=%li_%li in %s at %li\n", myID, x, memelapsed, (long) repeats, (long) size, (long)(repeats * size), totalsize, file, line);
+  fprintf(memfile,"%li %li %li %s\n",(long) repeats*newsize, totalsize, line, file);
+  //    fprintf(memfile,"%i> %p %.9f . calloc:%lix%li  size=%li_%li in %s at %li\n", myID, x, memelapsed, (long) repeats, (long) size, (long)(repeats * size), totalsize, file, line);
 #endif    
     if (x == NULL)
     {
@@ -598,7 +601,8 @@ LAMARC_realloc (void *ptr, size_t size, const char file[],
     (memt_finish.tv_usec - memt_start.tv_usec) / 1.e6;
     newsize = malloc_size(x);
     totalsize += newsize - oldsize;
-    fprintf(memfile,"%i> %p %.9f %p realloc size=%li_%li in %s at %li\n",myID, x, memelapsed, ptr, (long) newsize, totalsize, file, line);
+    fprintf(memfile,"%li %li %li %s\n",(long) newsize-oldsize, totalsize, line, file);
+    //fprintf(memfile,"%i> %p %.9f %p realloc size=%li_%li in %s at %li\n",myID, x, memelapsed, ptr, (long) newsize, totalsize, file, line);
 #endif    
     
     if (x == NULL)
