@@ -2,12 +2,12 @@
  inference of population parameters
  using a Metropolis-Hastings Monte Carlo algorithm
  -------------------------------------------------------
- speciation (fusion and fission routines)
+ speciation (fusion [and later fission] routines)
  
- Peter Beerli 2013, Seattle
+ Peter Beerli 2013, Tallahassee
  beerli@fsu.edu
  
- Copyright 2013 Peter Beerli
+ Copyright 2013-2015 Peter Beerli
  
  Permission is hereby granted, free of charge, to any person obtaining
  a copy of this software and associated documentation files (the
@@ -51,85 +51,8 @@ speciation tools
 
 extern profuncptr *propose_new;
 
-#ifdef AVX_not_working
-#include <immintrin.h>
-void myerf(double *x, double *result)
-{
-  __m256d v1 = _mm256_loadu_pd(x);
-  __m256d yy = _mm256_erf_pd(v1);
-  _mm256_storeu_pd(result,yy);
-}
-#endif
 
-
-
-// Abramovitz and Stegun 7.1.26
-// gain on Mac seems about 1/3
-double myerf(double x)
-{
-  const double p = 0.3275911;
-  const double a1= 0.254829592;
-  const double a2= -0.284496736;
-  const double a3=1.421413741;
-  const double a4 = -1.453152027;
-  const double a5 = 1.061405429;
-  const double t = 1.0 / (1.0 + p*x);
-  const double t2 = t * t;
-  const double t3 = t * t2;
-  const double t4 = t2 * t2;
-  const double t5 = t2 * t3;
-  return 1.0 - (a1*t + a2*t2 + a3*t3 + a4*t4 + a5*t5)*exp(-(x*x));
-}
-double myerfc(double x)
-{
-  const double p = 0.3275911;
-  const double a1= 0.254829592;
-  const double a2= -0.284496736;
-  const double a3=1.421413741;
-  const double a4 = -1.453152027;
-  const double a5 = 1.061405429;
-  const double t = 1.0 / (1.0 + p*x);
-  const double t2 = t * t;
-  const double t3 = t * t2;
-  const double t4 = t2 * t2;
-  const double t5 = t2 * t3;
-  return (a1*t + a2*t2 + a3*t3 + a4*t4 + a5*t5)*exp(-(x*x));
-}
-
-
-double myerf1(double x)
-{
-  const double p = 0.47047;
-  const double a1= 0.3480242;
-  const double a2= -0.0958798;
-  const double a3=0.7478556;
-  double t = 1.0 / (1.0 + p*x);
-  double t2 = t * t;
-  double t3 = t * t2;
-  return 1.0 - (a1*t + a2*t2 + a3*t3)*exp(-(x*x));
-}
-
-double myerfc1(double x)
-{
-  const double p = 0.47047;
-  const double a1= 0.3480242;
-  const double a2= -0.0958798;
-  const double a3=0.7478556;
-  double t = 1.0 / (1.0 + p*x);
-  double t2 = t * t;
-  double t3 = t * t2;
-  return (a1*t + a2*t2 + a3*t3)*exp(-(x*x));
-}
-
-
-#ifdef FASTERF
-#define ERF myerf
-#define ERFC myerfc
-#else
-#define ERF erf
-#define ERFC erfc
-#endif
-MYREAL eventtime_single(world_fmt *world, long pop, long timeslice, long *lineages, double age, char * event, long *to, long *from);
+MYREAL eventtime_single(proposal_fmt *proposal, world_fmt *world, long pop, long timeslice, long *lineages, double age, char * event, long *to, long *from);
 
 int beyond_last_node(proposal_fmt* proposal, vtlist *tentry, long gte, long *slider);
 
@@ -348,41 +271,11 @@ species_fmt * get_species_model(long to, species_fmt * s , long ssize)
 // this assumes that the speciation time follows a normal distribution
 // with mean mu and standard deviation sigma, but mu and sigma
 // are drawn from a prior 
-MYREAL log_prob_wait_speciate_old(MYREAL t0, MYREAL t1, MYREAL mu, MYREAL sigma, species_fmt *s)
-{
-  const MYREAL d = (sqrt(2.0) * sigma);
-  MYREAL x0 = (mu - t0)/d;
-  MYREAL x1 = (mu - t1)/d;
-  MYREAL e0 = 1.0 + ERF(x0);
-  MYREAL e1 = 1.0 + ERF(x1);
-  if (e1 == 0.0)
-    return -HUGE;
-  else
-    return -log(e0/e1);
-}
-
 // mathematica integral over t0 to t1 for truncated normal between 0 and b1 for mu and sigma
 // returns -integral(hazardfunction(truncated(normal(mu,sigma),{0,b1}))
 // -Log[Erf[(a - b1)/(Sqrt[2] b)] - Erf[(a - tau0)/(Sqrt[2] b)]] + 
 //     Log[Erf[(a - b1)/(Sqrt[2] b)] - Erf[(a - tau1)/(Sqrt[2] b)]]
-MYREAL log_prob_wait_speciate_truncold(MYREAL t0, MYREAL t1, MYREAL mu, MYREAL sigma, species_fmt *s)
-{
-  const MYREAL qsd = 1./(sqrt(2.0) * sigma);
-  const MYREAL b1 = s->max;
-  if (0.0 > t0)
-    return -HUGE;
-  if(t1 > b1)
-    return -HUGE;
-  MYREAL eb1 = ERF((mu - b1) * qsd);
-  MYREAL x0 = (mu - t0) * qsd;
-  MYREAL x1 = (mu - t1) * qsd;
-  MYREAL e0 = eb1 - ERF(x0);
-  MYREAL e1 = eb1 - ERF(x1);
-  if (e0 == 0.0)
-    return -HUGE;
-  else
-    return log(e1/e0);
-}
+//4.1.4
 MYREAL log_prob_wait_speciate(MYREAL t0, MYREAL t1, MYREAL mu, MYREAL sigma, species_fmt *s)
 {
   const MYREAL qsd = 1./(sqrt(2.0) * sigma);
@@ -391,34 +284,75 @@ MYREAL log_prob_wait_speciate(MYREAL t0, MYREAL t1, MYREAL mu, MYREAL sigma, spe
     return -HUGE;
   if(t1 > b1)
     return -HUGE;
-  MYREAL eb1 = ERF((b1 - mu) * qsd);
+  MYREAL eb1 = ERF((b1 - mu) * qsd); 
   MYREAL x0 = (mu - t0) * qsd;
   MYREAL x1 = (mu - t1) * qsd;
   MYREAL e0 = eb1 + ERF(x0);
   MYREAL e1 = eb1 + ERF(x1);
-  if ((e0 == 0.0) || (e1 == 0.0))
+  //printf ("@ t1=%f e0=%f e1=%f r=%f\n", t1, e0,e1, e1/e0);
+  if (e0 == 0.0)
     return -HUGE;
   else
-    return log(e1) - log(e0);
+    return log(e1/e0);
 }
-
+// 4.2.1 
+MYREAL log_prob_wait_speciate_421(MYREAL t0, MYREAL t1, MYREAL mu, MYREAL sigma, species_fmt *s)
+{
+  MYREAL xx = -HUGE;
+  const MYREAL qsd = 1./(SQRT2 * sigma);
+  const MYREAL b1 = s->max;
+  if (0.0 > t0)
+    return -HUGE;
+  if(t1 > b1)
+    return -HUGE;
+  MYREAL eb1 = -ERF((b1 - mu) * qsd);
+  MYREAL x0 = (mu - t0) * qsd;
+  MYREAL x1 = (mu - t1) * qsd;
+  MYREAL e0 = eb1 + ERF(x0); // eb1>0 if mu < b1 
+  MYREAL e1 = eb1 + ERF(x1); // ERF(x) > 0 if x>0, if t>mu && ERF(x)<eb1
+  if (e1 == 0.0)  
+    e1 = VERYSMALL_VALUE;
+  if ((e0 < 0.0 && e1 < 0.0) || (e0 > 0.0 && e1 > 0.0))
+    xx = LOG(e0/e1);
+  return xx;
+}
+//4.1.4
 MYREAL log_point_prob_speciate(MYREAL t1, MYREAL mu, MYREAL sigma, species_fmt * s)
 {
   const MYREAL b1 = s->max;
   const MYREAL qsd = 1./(sqrt(2.0) * sigma);
-  const MYREAL mut = (mu - t1);
+  const MYREAL mut = (t1 - mu);
+  const MYREAL mutt = (mu - t1);
   const MYREAL mut2 = mut*mut;
   const MYREAL var = sigma*sigma;
   // using a truncated normal (0..b1)
-  // old
-  // -((-a + x)^2/(2 b^2)) - Log[b] + 1/2 (-Log[2] - Log[\[Pi]]) - 
-  //    Log[-(1/2) Erfc[a/(Sqrt[2] b)] + 1/2 Erfc[(a - b1)/(Sqrt[2] b)]]
-  // new
-  //-((mu - t)^2/(2 sigma^2)) + 1/2 (Log[2] - Log[\[Pi]]) - Log[sigma] - 
+  //-((mu - t)^2/(2 sigma^2)) + 1/2 (-Log[2] - Log[\[Pi]]) - Log[sigma] - 
   //Log[Erfc[(-b1 + mu)/(Sqrt[2] sigma)] - 
   //    Erfc[(mu - t)/(Sqrt[2] sigma)]]
-  MYREAL prob =  -(mut2/(2 * var)) - log(sigma) + LOG2MINUSPIHALF 
-    - log(ERFC((mu - b1)*qsd) - ERFC(mu*qsd));
+  MYREAL prob =  -(mut2/(2 * var)) + LOG2PIHALF - log(sigma) 
+    - log(0.5 * ERFC((mu - b1)*qsd) - 0.5 * ERFC(mutt*qsd));
+  return prob;
+}
+//4.2.1
+MYREAL log_point_prob_speciate_421(MYREAL t1, MYREAL mu, MYREAL sigma, species_fmt * s)
+{
+  //  -((mu - t1)^2/(2 sigma^2)) + 1/2 (Log[2] - Log[\[Pi]]) - 
+  // Log[sigma] - Log[1/2 Erfc[(-b1 + mu)/(Sqrt[2] sigma)] - 
+  //    1/2 Erfc[(mu - t1)/(Sqrt[2] sigma)]]
+  //const MYREAL b0 = s->min;
+  const MYREAL b1 = s->max;
+  const MYREAL var = sigma*sigma;
+  const MYREAL qsd = 1./ (SQRT2 * sigma);
+  const MYREAL qsd2 = 1. / (2.0 * var);
+  const MYREAL mut = (mu - t1);
+  const MYREAL mub = (mu - b1);
+  const MYREAL mut2 = mut*mut;
+
+  MYREAL xx = 0.5 * (ERFC(mub*qsd) - ERFC(mut*qsd));
+  if (xx == 0.0)
+    xx = VERYSMALL_VALUE;  
+  xx = LOG(xx);
+  MYREAL prob =  -(mut2*qsd2) - log(sigma) + LOG2PIHALF  - xx; 
   return prob;
 }
 
@@ -464,8 +398,67 @@ MYREAL set_type(world_fmt *world, long topop, long frompop, char *custm2, long n
   return type;
 }
 
-
+//invert the speciation probability to get the time of event of split
+// getprob[mu_, std_, b1_, t0_, t1_] := 
+//                   Log[Erf[(-b1 + mu)/(Sqrt[2] std)] - Erf[(mu - t1)/(Sqrt[2] std)] ] - 
+//                   Log[Erf[(-b1 + mu)/(Sqrt[2] std)] - Erf[(mu - t0)/(Sqrt[2] std)]]
+// In: Solve[Log[r] == getprob[mu, std, b1, t0, t1], t1]
+// Out: t1 <- mu - Sqrt[2] std InverseErf[(-1 + r) Erf[(b1 - mu)/(Sqrt[2] std)] + 
+//		       r Erf[(mu - t0)/(Sqrt[2] std)]]
 MYREAL time_to_speciate(world_fmt *world, long pop, MYREAL t0, char *event, long *to, long *from)
+{
+  long ssize = world->species_model_size;
+  species_fmt *s = get_species_model(pop, world->species_model, ssize);
+  MYREAL r;
+  MYREAL tmax;
+  //MYREAL tmin = t0;
+  MYREAL t1 ;
+  long paramindex;
+  MYREAL mu;
+  MYREAL sigma;
+  double qsd;
+  MYREAL interval;
+  if(s==NULL)
+    return HUGE;
+  paramindex = world->numpop2 + world->bayes->mu + 2 * s->id;
+  mu = world->param0[paramindex];
+  sigma = world->param0[paramindex+1];
+  tmax = s->max;
+  qsd = 1./(SQRT2 * sigma);
+  r = UNIF_RANDUM ();
+  // we have a good implementation of the inverse cumulative normal and use the scaling to
+  // to get the inverse error function, and scale for the time, return t1
+  // p[mu_, std_, t0_, b1_] := 
+  // Plot[{  mu - std  InverseCDF[
+  // 	    NormalDistribution[0, 
+  // 			       1], (((-1 + x) Erf[(b1 - mu)/(Sqrt[2] std)] + 
+  // 				     x Erf[(mu - t0)/(Sqrt[2] std)]) + 1)/2], 
+  // 	    mu - Sqrt[2]std  InverseErf[(-1 + x) Erf[(b1 - mu)/(Sqrt[2] std)] + 
+  // 		    x Erf[(mu - t0)/(Sqrt[2] std)]]}, {x, 0, 1}, 
+  //  PlotStyle -> {{Blue, Thick}, {Dashed}}] 
+  //  p[1, 0.2, 0, 5]
+  t1 =  mu - sigma * inverse_cumstd_normal((((r-1.0) * ERF((tmax - mu) * qsd) +  r * ERF((mu-t0)*qsd))+1.0)/2.0);
+  //printf("@t1: t0=%f t1=%f\n",t0,t1); 
+
+  *event = 'd';
+  *from = s->from;
+  *to = s->to;//should be the same as pop
+
+  if (t0 <= t1)// && t1 < tmax)
+    {
+      interval =  t1-t0;  
+      return interval;
+    }
+  else
+    {
+      if (t1 - t0 > -10e-8)
+	return 0.0;
+      else
+	return HUGE;
+    }
+}
+//@@@@@@@@@@@@@@@@@@@@@@@@@
+MYREAL time_to_speciate_halfing(world_fmt *world, long pop, MYREAL t0, char *event, long *to, long *from)
 {
   long ssize = world->species_model_size;
   species_fmt *s = get_species_model(pop, world->species_model, ssize);
@@ -491,11 +484,11 @@ MYREAL time_to_speciate(world_fmt *world, long pop, MYREAL t0, char *event, long
     {
       *event = 'd';
       *from = s->from;
-      *to = s->to;//should be the same as pop
-      return 1e-10;
+      *to = s->to; //should be the same as pop
+      return 1e10;
     }
   t1 = (tmax+tmin)/2.;
-  //check_speciateparameters(s);
+  // check_speciateparameters(s);
   r = LOG(UNIF_RANDUM ());
   lambda = log_prob_wait_speciate(t0, t1 , mu, sigma, s) - r;
   while (fabs(lambda)>EPSILON && counter <= 1000 && (tmax-tmin)> EPSILON)
@@ -511,11 +504,10 @@ MYREAL time_to_speciate(world_fmt *world, long pop, MYREAL t0, char *event, long
 	}
       t1 = (tmin+tmax)/2.0;
       //
-
-      //printf("@@ %i> %li t0=%f t1=%f r=%f lambda=%f\n",myID,counter, t0,t1,r,lambda);
-      lambda = log_prob_wait_speciate(t0, t1 , mu,sigma,s)-r;
+      lambda = log_prob_wait_speciate(t0, t1 , mu,sigma,s) - r;
+      //printf("%i> @    @@@  %li t0=%f t1=%f r=%f lambda=%f (%f)\n",myID,counter, t0,t1,r,lambda, lambda+r);
     }  
-  //printf("%i> D %li t0=%f t1=%f r=%f lambda=%f\n",myID,counter, t0,t1,r,lambda);
+  //printf("%i> @@@@D %li t0=%f t1=%f t1-t0=%f r=%f lambda=%f\n",myID,counter, t0,t1,t1-t0, r,lambda);
 
   if (counter>=1000)
     t1=HUGE;
@@ -661,7 +653,7 @@ MYREAL time_to_coalescence(world_fmt * world, long pop, long timeslice, long *li
   return interval;
 }
 
-MYREAL time_to_migration(world_fmt *world, long pop, long timeslice, long * lineages, char * event, long * to, long * from)
+MYREAL time_to_migration(proposal_fmt *proposal, world_fmt *world, long pop, long timeslice, long * lineages, char * event, long * to, long * from)
 {
   long    addition=0;
   long    numpop = world->numpop;
@@ -687,6 +679,17 @@ MYREAL time_to_migration(world_fmt *world, long pop, long timeslice, long * line
   long fromx = *from;
   long the_to = *to;
   long the_from = *from;
+  long offlimit_from = -1; //nothing is offlimit
+  if(proposal!=NULL 
+     && proposal->migr_table_counter>0 
+     && proposal->migr_table[proposal->migr_table_counter-1].event=='d')
+    {
+      // if we have encountered a divergence then we will not get a migration
+      // back to the other population: e.g. D: from=1 to=2 --> no M from 2-->1
+      // for the lineages proposed: custom={*D**} eseentially results for this 
+      // lineage into {*d**}
+      offlimit_from = proposal->migr_table[proposal->migr_table_counter-1].to; 
+    }
   for (i = msta; i < msto; i++)
     { 
       switch (custm2[i])
@@ -702,9 +705,19 @@ MYREAL time_to_migration(world_fmt *world, long pop, long timeslice, long * line
 	      warning("skyparam[%li]=%f rate=%f\n",i, skyparam[i],rate);
 	      error("mismatch with skyparams\n");
 	    }
-	  mm =  world->data->geo[i] * world->param0[i] * skyparam[i]/rate;
-	  invdenom = 1.0 / (mm);
-	  interval =  (-(LOG (UNIF_RANDUM ())) * invdenom) ;
+	  long frompop = i - numpop - (pop) * (numpop - 1);
+	  if (frompop >= pop)
+	    frompop += 1;
+	  if(offlimit_from == frompop)
+	    {
+	      interval=HUGE;
+	    }
+	  else
+	    {
+	      mm =  world->data->geo[i] * world->param0[i] * skyparam[i]/rate;
+	      invdenom = 1.0 / (mm);
+	      interval =  (-(LOG (UNIF_RANDUM ())) * invdenom) ;
+	    }
 	  if (the_eventtime > interval)
 	    {
 	      the_eventtime = interval;
@@ -764,7 +777,7 @@ void keep_min_eventtime(MYREAL *the_eventtime, MYREAL eventtime, char *the_event
     }
 }
 
-MYREAL eventtime_single(world_fmt *world, long pop, long timeslice, long *lineages, double age, char * event, long *to, long *from)
+MYREAL eventtime_single(proposal_fmt *proposal, world_fmt *world, long pop, long timeslice, long *lineages, double age, char * event, long *to, long *from)
 {
   char the_event = ' ';
   MYREAL the_eventtime=HUGE;
@@ -775,10 +788,17 @@ MYREAL eventtime_single(world_fmt *world, long pop, long timeslice, long *lineag
   //eventtime = time_to_coalmig(world, pop, timeslice, lineages, &myevent,&tox, &fromx);
   eventtime = time_to_coalescence(world, pop, timeslice, lineages, &myevent,&tox);
   keep_min_eventtime(&the_eventtime, eventtime, &the_event, myevent, to, tox, from, fromx);
-  eventtime = time_to_migration(world, pop, timeslice, lineages, &myevent, &tox, &fromx);
+  //if(proposal!=NULL && proposal->migr_table_counter>0 && proposal->migr_table[proposal->migr_table_counter-1].event=='d')
+  //{
+  //  eventtime = HUGE;
+  //}
+  //else
+  //  { 
+  eventtime = time_to_migration(proposal,world, pop, timeslice, lineages, &myevent, &tox, &fromx);
   keep_min_eventtime(&the_eventtime, eventtime, &the_event, myevent, to, tox, from, fromx);
   eventtime = time_to_speciation(world, pop, age, &myevent, &tox, &fromx);
   keep_min_eventtime(&the_eventtime, eventtime, &the_event, myevent, to, tox, from, fromx);
+  //  }
   *event = the_event;
   if (*event == 'm' && *from == *to)
     error("FAIL");
@@ -796,7 +816,8 @@ long speciation_from(long to, proposal_fmt * proposal)
       if (to == s[i].to)
 	return s[i].from;
     }
-  error("there should be a species 'from'");
+  warning("Inclusion of a population splitting event failed\n");
+  warning("because target was not found, this sample was ignored, but run continues\n");
   return -1;
 }
 
@@ -837,6 +858,9 @@ newtree_update (world_fmt * world, long g, boolean assign)
     timelist_fmt *timevector = NULL; /* local timelist */
     vtlist *tentry = NULL; /*pointer into timeslice */
     MYREAL x;
+    long i, ii;
+    long np = world->numpop2 + world->bayes->mu + world->species_model_size * 2 ;
+
     if (assign && world->unassignednum<2)
       return 0;
 
@@ -846,6 +870,14 @@ newtree_update (world_fmt * world, long g, boolean assign)
     if(assign)
       {
 	chooseUnassigned (proposal);
+	if (proposal->origin == NULL)
+	  {
+	    free_timevector(timevector);
+#ifndef TESTING2
+	    free_masterproposal (proposal);
+#endif	    
+	    return 0;
+	  }
 	oldpop = proposal->origin->actualpop;
 	newpop = oldpop;
 	if (world->numpop>1)
@@ -870,6 +902,16 @@ newtree_update (world_fmt * world, long g, boolean assign)
     }
 
     construct_localtimelist (timevector, proposal);
+    if(proposal->divlist->div_elem>0)
+      {
+	memcpy(proposal->param0save, proposal->world->param0, sizeof(MYREAL)*np);
+	for (i=0;i<proposal->divlist->div_elem;i++)
+	  {
+	    ii = m2mmm(proposal->divlist->divlist[i][0],
+		       proposal->divlist->divlist[i][1],proposal->world->numpop);
+	    proposal->world->param0[ii]=0.0;
+	  }
+      }
     tentry = &(*timevector).tl[0];
     age = proposal->origin->tyme;
     if(age<0.0)
@@ -914,11 +956,11 @@ newtree_update (world_fmt * world, long g, boolean assign)
          0) ? proposal->migr_table[proposal->migr_table_counter -
                                    1].from : proposal->origin->pop;
 
-	x = eventtime_single(world, actualpop, tentry->timeslice, tentry->lineages, tentry->age, &event, &to, &from);
+	x = eventtime_single(proposal,world, actualpop, tentry->timeslice, tentry->lineages, tentry->age, &event, &to, &from);
 	int count=0;
 	while(x < 0.0 && count++ < 100)
 	  {
-	    x = eventtime_single(world, actualpop, tentry->timeslice, tentry->lineages, tentry->age, &event, &to, &from);
+	    x = eventtime_single(proposal,world, actualpop, tentry->timeslice, tentry->lineages, tentry->age, &event, &to, &from);
 	    printf("%i> event=%c to=%li from=%li x=%f age+x=%f nexttime%f\n",myID,event,to, from, x, age+x,nexttime);
 	  }
 	//printf("%i> event=%c to=%li from=%li x=%f age+x=%f nexttime%f\n",myID,event,to, from, x, age+x,nexttime);
@@ -946,16 +988,19 @@ newtree_update (world_fmt * world, long g, boolean assign)
 		  loopcleanup(assign, world, proposal, oldpop, timevector);
 		  return 0;
                 }
-                age = proposal->time;
-		//printf("%c: to=%li from=%li time=%f\n", event,
-		//       proposal->migr_table[proposal->migr_table_counter - 1].to,
-		//       proposal->migr_table[proposal->migr_table_counter - 1].from,
-		//       proposal->migr_table[proposal->migr_table_counter - 1].time);
-                continue;
+	      age = proposal->time;
+	      //printf("%c: to=%li from=%li time=%f\n", event,
+	      //       proposal->migr_table[proposal->migr_table_counter - 1].to,
+	      //       proposal->migr_table[proposal->migr_table_counter - 1].from,
+	      //       proposal->migr_table[proposal->migr_table_counter - 1].time);
+	      continue;
             }
             else
             {   /*coalesce */
-	      //printf("%c: pop=%li time=%f\n", event, actualpop, proposal->time); 
+
+	      //printf("%i> %c: pop=%li time=%f\n", myID, event, actualpop, proposal->time); 
+	      //if (event != 'c')
+	      //	error("Abort in new_treeupdate(), line 1024");
 	      //printf("**\n");
                 chooseTarget (proposal, timevector, proposal->bordernodes,
                               &bordernum);
@@ -970,6 +1015,10 @@ newtree_update (world_fmt * world, long g, boolean assign)
                     age = proposal->time;
                     continue;
                 }
+		if(proposal->divlist->div_elem>0)
+		  {
+		    memcpy(proposal->world->param0, proposal->param0save,  sizeof(MYREAL)*np);
+		  }
                 pretendcoalesce1p (proposal);
                 coalesced = TRUE;
                 break;
@@ -1004,6 +1053,10 @@ newtree_update (world_fmt * world, long g, boolean assign)
 	  loopcleanup(assign, world, proposal, oldpop, timevector);
 	  return 0;
         }
+	if(proposal->divlist->div_elem>0)
+	  {
+	    memcpy(proposal->world->param0, proposal->param0save,  sizeof(MYREAL)*np);
+	  }
         pretendcoalesce1p (proposal);
     }
     if (acceptlike (world, proposal, g, timevector))
@@ -1012,6 +1065,29 @@ newtree_update (world_fmt * world, long g, boolean assign)
         {   /*saveguard */
             world->root->tyme += proposal->time;
         }
+#ifdef DEBUGXX
+	int i;
+	for(i=0;i<proposal->migr_table_counter; i++)
+	  {
+	    if (proposal->migr_table[i].event=='d')
+	      {
+		printf("d ");
+		if (i!=proposal->migr_table_counter-1)
+		  error("DARN");
+	      }
+	    printf("%i> PMT:  %li %li, %f %c\n",myID,proposal->migr_table[i].from,proposal->migr_table[i].to,proposal->migr_table[i].time,proposal->migr_table[i].event);
+	  }
+	for(i=0;i<proposal->migr_table_counter2; i++)
+	  {	    if (proposal->migr_table[i].event=='d')
+	      {
+		printf("d2 ");
+		if (i!=proposal->migr_table_counter-1)
+		  error("DARN2");
+	      }
+	    printf("%i> PMT2: %li %li, %f %c\n",myID,proposal->migr_table2[i].from,proposal->migr_table2[i].to,proposal->migr_table2[i].time,proposal->migr_table2[i].event);
+	  }
+	printf("\n");
+#endif
         coalesce1p (proposal);
 	if(world->cold && assign)
 	  {
@@ -1033,7 +1109,7 @@ newtree_update (world_fmt * world, long g, boolean assign)
         count_migrations (world->root->next->back, &world->migration_counts);
         free_timevector (timevector);
 #ifndef TESTING2
-            free_masterproposal (proposal);
+	free_masterproposal (proposal);
 #endif
         return 1;   /* new tree accepted */
       }
@@ -1047,6 +1123,8 @@ newtree_update (world_fmt * world, long g, boolean assign)
 	loopcleanup(assign, world, proposal, oldpop, timevector);
 	return 0;
       }
+    error("do not go here in new_tree_update");
+    return -1;
 }
 
 void set_things(MYREAL t1, MYREAL t2, char e1, char e2, long to1, long from1, long to2, long from2, 
@@ -1070,15 +1148,17 @@ void set_things(MYREAL t1, MYREAL t2, char e1, char e2, long to1, long from1, lo
 
 // calculates the time=age and event for each lineage (of the final 2) 
 // in beyond_....()
-void calc_time_per_line(proposal_fmt * proposal, boolean same,MYREAL *time, char *event, long *to, long *from)
+void calc_time_per_line(proposal_fmt * proposal, char type, long popto, boolean same,MYREAL *time, char *event, long *to, long *from)
 {
   long pop1 = *to;
   MYREAL age1 = *time;
   MYREAL r0, r1;
-  long froms, tos;
+  long froms = *from;
+  long tos = *to;
   char events;
   MYREAL time_coal, time_mig, time_spec;
   world_fmt * w = proposal->world;
+  const long numpop = w->numpop;
   MYREAL  rate = w->options->mu_rates[w->locus];
   //MYREAL  invrate = 1./rate; 
   MYREAL  *skyparam;
@@ -1124,6 +1204,14 @@ void calc_time_per_line(proposal_fmt * proposal, boolean same,MYREAL *time, char
   // generate now times for each possible migration j->pop1, and also speciation pattern i->pop1
   for (i = msta; i < msto; i++)
     {
+      if (type=='d')
+	{
+	  long frompop = i - numpop - (pop1) * (numpop - 1);
+	  if (frompop >= pop1)
+            frompop += 1;
+	  if (frompop==popto) // (i-numpop)
+	    continue;
+	}
       // if the combintion j->pop1 is not a speciation type event
       //1229if (w->options->custm2[i] != 'd')
       //1229{
@@ -1147,7 +1235,7 @@ void calc_time_per_line(proposal_fmt * proposal, boolean same,MYREAL *time, char
     }
   // assumes no population fusion only splitting
   // revisit this!
-  time_spec = time_to_speciate(w, pop1, age1, &events, &tos, &froms);
+  time_spec = time_to_speciation(w, pop1, age1, &events, &tos, &froms);
   set_things(*time, age1 + time_spec, *event, 'd', *to, *from, tos, froms, time, event, to, from);
 }
 
@@ -1160,14 +1248,14 @@ void calc_time_per_line(proposal_fmt * proposal, boolean same,MYREAL *time, char
 int beyond_last_node(proposal_fmt* proposal, vtlist *tentry, long gte, long *slider)
 {
   long pop1, pop2;
+  char type1;
+  char type2;
+  long popto1, popto2;
   MYREAL age1, horizon;
   char event;
   MYREAL time;
   long to, from;
   long pmc1, pmc2;
-  //world_fmt * w = proposal->world;
-  //boolean not_in_ancestor1;
-  //boolean not_in_ancestor2;
   boolean dangling;
   MYREAL time1, time2;
   char event1, event2;
@@ -1197,6 +1285,8 @@ int beyond_last_node(proposal_fmt* proposal, vtlist *tentry, long gte, long *sli
   pop2 = proposal->realtarget->pop;
   pop1 = proposal->migr_table_counter > 0 ? proposal->migr_table[proposal->migr_table_counter -
 								 1].from : proposal->origin->pop;
+  type1 = proposal->migr_table_counter > 0 ? proposal->migr_table[proposal->migr_table_counter -
+								  1].event : proposal->origin->type; 
   age1 = MAX (proposal->realtarget->tyme,
 	      proposal->migr_table_counter >
 	      0 ? proposal->migr_table[proposal->migr_table_counter -
@@ -1208,15 +1298,6 @@ int beyond_last_node(proposal_fmt* proposal, vtlist *tentry, long gte, long *sli
   // lineages have not crossed into the ancestor yet.
   // one population
   event = ' ';
-  //if(get_species_model(pop1, w->species_model, w->species_model_size)!=NULL)
-  //  {
-  //    not_in_ancestor1 = TRUE;
-  //  }
-  //if(get_species_model(pop2, w->species_model, w->species_model_size)!=NULL)
-   // {
-	//  not_in_ancestor2 = TRUE;
-   // }
-  // start
   // pop1 is the actual population at proposal->time in the dangling lineage
   // pop2 is the last lineage in the residual tree 
   time = age1;
@@ -1229,9 +1310,13 @@ int beyond_last_node(proposal_fmt* proposal, vtlist *tentry, long gte, long *sli
       pop2 = proposal->realtarget->pop;
       pop1 = proposal->migr_table_counter > 0 ? proposal->migr_table[proposal->migr_table_counter -
 								     1].from : proposal->origin->pop;
+      type1 = proposal->migr_table_counter > 0 ? proposal->migr_table[proposal->migr_table_counter -
+								      1].event : proposal->origin->type;
+      popto1 = proposal->migr_table_counter > 0 ? proposal->migr_table[proposal->migr_table_counter -
+								       1].to : proposal->origin->actualpop;
       // calculate only for the dangling lineage, gets the timeinterval (not the full age)
       to = pop1;
-      calc_time_per_line(proposal, pop1 == pop2, &time, &event, &to, &from);
+      calc_time_per_line(proposal, type1, popto1, pop1 == pop2, &time, &event, &to, &from);
       // start proposal->time pop1, pop2
       // proposal time        to/from pop2
       if (time >= horizon)
@@ -1260,12 +1345,16 @@ int beyond_last_node(proposal_fmt* proposal, vtlist *tentry, long gte, long *sli
       time1 = time2 = proposal->time;
       pmc1 = proposal->migr_table_counter;
       pop1 = pmc1 > 0 ? proposal->migr_table[pmc1 - 1].from : proposal->origin->pop;
+      type1= pmc1 > 0 ? proposal->migr_table[pmc1 - 1].event : proposal->origin->type;
+      popto1= pmc1 > 0 ? proposal->migr_table[pmc1 - 1].to : proposal->origin->actualpop;
       pmc2 = proposal->migr_table_counter2;
-      pop2 = pmc2 > 0 ? proposal->migr_table2[pmc2 - 1].from : proposal->realtarget->pop;      
+      pop2 = pmc2 > 0 ? proposal->migr_table2[pmc2 - 1].from : proposal->realtarget->pop;
+      type2= pmc2 > 0 ? proposal->migr_table2[pmc2 - 1].event : proposal->realtarget->type;
+      popto2= pmc2 > 0 ? proposal->migr_table2[pmc2 - 1].to : proposal->realtarget->actualpop;
       to1 = pop1;
       to2 = pop2;
-      calc_time_per_line(proposal, pop1 == pop2, &time1, &event1, &to1, &from1);
-      calc_time_per_line(proposal, pop1 == pop2, &time2, &event2, &to2, &from2);
+      calc_time_per_line(proposal, type1, popto1, pop1 == pop2, &time1, &event1, &to1, &from1);
+      calc_time_per_line(proposal, type2, popto2, pop1 == pop2, &time2, &event2, &to2, &from2);
       //printf("%i> _both() interval line1: (%f,%c) line 2: (%f, %c) \n",myID,time1 - proposal->time, event1,time2 - proposal->time, event2);
       if (time1 < time2)
 	{
@@ -1283,6 +1372,8 @@ int beyond_last_node(proposal_fmt* proposal, vtlist *tentry, long gte, long *sli
 	  to = to2;
 	  from = from2;
 	}
+      if (time >= 1e10)// HUGE, but HUge seems not to catch all DEBUG TODO PROBLEM
+	return 0;
       proposal->time = time;
       if (event == 'c')
 	{
@@ -1297,15 +1388,15 @@ int beyond_last_node(proposal_fmt* proposal, vtlist *tentry, long gte, long *sli
 	    }
 	  //pmc1 = proposal->migr_table_counter;
 	  //pop1 = pmc1 > 0 ? proposal->migr_table[pmc1 - 1].from : proposal->origin->pop;
-        }
+	}
       else
         {
 	  if (!migrateb (proposal, proposal->realtarget, event))
-            {
+	    {
 	      return 0;  /* migration limit reached */
-            }
-	  //pmc2 = proposal->migr_table_counter2;
-	  //pop2 = pmc2 > 0 ? proposal->migr_table2[pmc2 - 1].from : proposal->realtarget->pop;
+	    }
+	      //pmc2 = proposal->migr_table_counter2;
+	      //pop2 = pmc2 > 0 ? proposal->migr_table2[pmc2 - 1].from : proposal->realtarget->pop;
 	}
     }
   return 0;
@@ -1349,7 +1440,7 @@ long propose_new_spec_mu(world_fmt * world, long which, boolean *is_mu, MYREAL *
       s->sigma = *newparam;
     }
 #ifdef DEBUG
-  //  printf("%i> propose_newmu_sigma: %li <- %li: specmu=%f (param[%li]=%f) sigma=%f mumin=%f mumax=%f\n",myID, s->to, s->from, s->mu,which,world->param0[which], s->sigma, s->min, s->max);
+  //printf("%i> propose_newmu_sigma: %li <- %li: specmu=%f (param[%li]=%f) sigma=%f mumin=%f mumax=%f\n",myID, s->to, s->from, s->mu,which,world->param0[which], s->sigma, s->min, s->max);
 #endif
   return s->id;
 }
@@ -1378,7 +1469,6 @@ MYREAL wait_event_species(world_fmt *world, vtlist *tli, MYREAL t0, MYREAL t1, b
 	continue;
       else
 	{
-	  // log_prob_wait_speciate already returns negative
 	  mu = world->param0[npp + 2* s->id];
 	  sigma = world->param0[npp + 2* s->id + 1];
 	  //if (mu<EPSILON)
@@ -1604,6 +1694,76 @@ void construct_locusspecies_histogram(world_fmt *world, long locus, MYREAL *mini
     }
 }
 
+void get_div_from_tree(node *root, divtime_fmt **div, long *divtime_alloc, long *divtime_num)
+{
+  if (*divtime_num >= *divtime_alloc - 1)
+    {
+      *divtime_alloc += HUNDRED;
+      *div = (divtime_fmt *) realloc(*div, sizeof(divtime_fmt) * *divtime_alloc);
+    }
+  if (root == NULL)
+    return;
+  switch(root->type)
+    {
+    case 't':
+      return;
+    case 'r':
+      get_div_from_tree(root->next->back, div, divtime_alloc, divtime_num);
+      break;
+    case 'i':
+      get_div_from_tree(root->next->back, div, divtime_alloc, divtime_num);
+      get_div_from_tree(root->next->next->back, div, divtime_alloc, divtime_num);
+      break;
+    case 'm':
+      get_div_from_tree(root->next->back, div, divtime_alloc, divtime_num);
+      break;
+    case 'd':
+      (*div)[*divtime_num].age = root->tyme;
+      (*div)[*divtime_num].from = root->pop;
+      (*div)[*divtime_num].to = root->actualpop;
+      (*divtime_num) += 1;
+      get_div_from_tree(root->next->back, div, divtime_alloc, divtime_num);
+      break;
+    default:
+      error("missing type information in get_div_from_tree");
+      break;
+    }
+}
+
+void record_parameters(world_fmt *world)
+{
+  //extract ages of coalescences, migrations, divergences
+  //
+  if (world->cold)
+    {
+      world->divtime_num = 0;
+      get_div_from_tree(world->root, &world->divtime, &world->divtime_alloc, &world->divtime_num);
+      if (world->divtime_num>0)
+	{
+#ifdef MPI
+	  send_divtime("%li\n",world->divtime_num);
+#else
+	  fprintf(world->divtimefile,"%li\n",world->divtime_num);
+#endif
+	  int i;
+	  species_fmt *s = NULL; 	 
+	  for (i=0; i<world->divtime_num; i++)
+	    {
+	      s = get_species_model(world->divtime[i].to, world->species_model, world->species_model_size); 
+#ifdef MPI
+	      send_divtime("%f %li %li %f %f\n",world->divtime[i].age,world->divtime[i].from,world->divtime[i].to, s->mu, s->sigma);
+#else
+	  fprintf(world->divtimefile,"%f %li %li %f %f\n",world->divtime[i].age,world->divtime[i].from,world->divtime[i].to, s->mu, s->sigma);
+#endif
+
+	    }
+	}
+    }
+  //save to file or send to master
+  // tree# age thetas1,2,3 Ms21 12 ...  Ds21 12.. Stds21 12 ...
+  // - better: tree# age paramID value
+  
+}
 
 /*void adjust_priorsfor species
 @@@@@@
